@@ -118,8 +118,8 @@ save_primary_selection (AddressbookComponent *addressbook_component)
 				 e_source_peek_uid (source), NULL);
 }
 
-static void
-load_primary_selection (AddressbookComponent *addressbook_component)
+static ESource *
+get_primary_source (AddressbookComponent *addressbook_component)
 {
 	AddressbookComponentPrivate *priv;
 	ESource *source;
@@ -133,16 +133,26 @@ load_primary_selection (AddressbookComponent *addressbook_component)
 	if (uid) {
 		source = e_source_list_peek_source_by_uid (priv->source_list, uid);
 		g_free (uid);
-	
-		e_source_selector_set_primary_selection (E_SOURCE_SELECTOR (priv->source_selector), source);
 	} else {
-		ESource *source;
-		
 		/* Try to create a default if there isn't one */
 		source = find_first_source (priv->source_list);
-		if (source)
-			e_source_selector_set_primary_selection (E_SOURCE_SELECTOR (priv->source_selector), source);
 	}
+
+	return source;
+}
+
+static void
+load_primary_selection (AddressbookComponent *addressbook_component)
+{
+	AddressbookComponentPrivate *priv;
+	ESource *source;
+	char *uid;
+
+	priv = addressbook_component->priv;
+
+	source = get_primary_source (addressbook_component);
+	if (source)
+		e_source_selector_set_primary_selection (E_SOURCE_SELECTOR (priv->source_selector), source);
 }
 
 /* Folder popup menu callbacks */
@@ -358,7 +368,7 @@ impl_requestCreateItem (PortableServer_Servant servant,
 
 	priv = addressbook_component->priv;
 
-	selected_source = e_source_selector_peek_primary_selection (E_SOURCE_SELECTOR (priv->source_selector));
+	selected_source = get_primary_source (addressbook_component);
 	if (!selected_source) {
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION, ex_GNOME_Evolution_Component_Failed, NULL);
 		return;
@@ -398,19 +408,7 @@ impl_requestCreateItem (PortableServer_Servant servant,
 static CORBA_boolean
 impl_upgradeFromVersion (PortableServer_Servant servant, short major, short minor, short revision, CORBA_Environment *ev)
 {
-	switch (major) {
-	case 1:
-		switch (minor) {
-		case 0:
-		case 2:
-		case 4:
-			addressbook_migrate (addressbook_component_peek ());
-			break;
-		}
-		break;
-	}
-	
-	return TRUE;
+	return addressbook_migrate (addressbook_component_peek (), major, minor, revision);
 }
 
 /* GObject methods.  */
@@ -480,8 +478,7 @@ addressbook_component_init (AddressbookComponent *component)
 
 	priv = g_new0 (AddressbookComponentPrivate, 1);
 
-	/* EPFIXME: Should use a custom one instead?  Also we should add
-	   addressbook_component_peek_gconf_client().  */
+	/* EPFIXME: Should use a custom one instead? */
 	priv->gconf_client = gconf_client_get_default ();
 
 	priv->source_list = e_source_list_new_for_gconf (priv->gconf_client,
@@ -511,6 +508,14 @@ addressbook_component_peek (void)
 	return component;
 }
 
+GConfClient*
+addressbook_component_peek_gconf_client (AddressbookComponent *component)
+{
+	g_return_val_if_fail (ADDRESSBOOK_IS_COMPONENT (component), NULL);
+
+	return component->priv->gconf_client;
+}
+
 const char *
 addressbook_component_peek_base_directory (AddressbookComponent *component)
 {
