@@ -589,29 +589,29 @@ addressbook_show_load_error_dialog (GtkWidget *parent, ESource *source, EBookSta
 
 	if (!strncmp (uri, "file:", 5)) {
 		label_string = 
-			_("We were unable to open this addressbook.  Please check that the\n"
+			_("We were unable to open this addressbook.  Please check that the "
 			  "path exists and that you have permission to access it.");
 	}
 	else if (!strncmp (uri, "ldap:", 5)) {
 		/* special case for ldap: contact folders so we can tell the user about openldap */
 #if HAVE_LDAP
 		label_string = 
-			_("We were unable to open this addressbook.  This either\n"
-			  "means you have entered an incorrect URI, or the LDAP server\n"
+			_("We were unable to open this addressbook.  This either "
+			  "means you have entered an incorrect URI, or the LDAP server "
 			  "is unreachable.");
 #else
 		label_string =
-			_("This version of Evolution does not have LDAP support\n"
-			  "compiled in to it.  If you want to use LDAP in Evolution\n"
-			  "you must compile the program from the CVS sources after\n"
-			  "retrieving OpenLDAP from the link below.\n");
+			_("This version of Evolution does not have LDAP support "
+			  "compiled in to it.  If you want to use LDAP in Evolution "
+			  "you must compile the program from the CVS sources after "
+			  "retrieving OpenLDAP from the link below.");
 		href = gnome_href_new ("http://www.openldap.org/", "OpenLDAP at http://www.openldap.org/");
 #endif
 	} else {
 		/* other network folders */
 		label_string =
-			_("We were unable to open this addressbook.  This either\n"
-			  "means you have entered an incorrect URI, or the server\n"
+			_("We were unable to open this addressbook.  This either "
+			  "means you have entered an incorrect URI, or the server "
 			  "is unreachable.");
 	}
 
@@ -682,13 +682,27 @@ typedef struct {
 	EBookCallback cb;
 	ESource *source;
 	gpointer closure;
+	guint cancelled : 1;
 } LoadSourceData;
 
+static void
+free_load_source_data (LoadSourceData *data)
+{
+	if (data->source)
+		g_object_unref (data->source);
+	g_free (data);
+}
+
 static void
 load_source_auth_cb (EBook *book, EBookStatus status, gpointer closure)
 {
 	LoadSourceData *data = closure;
 
+	if (data->cancelled) {
+		free_load_source_data (data);
+		return;
+	}
+
 	if (status != E_BOOK_ERROR_OK) {
 		if (status == E_BOOK_ERROR_CANCELLED) {
 			/* the user clicked cancel in the password dialog */
@@ -701,7 +715,7 @@ load_source_auth_cb (EBook *book, EBookStatus status, gpointer closure)
 			g_signal_connect (dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
 			gtk_widget_show (dialog);
 			data->cb (book, E_BOOK_ERROR_OK, data->closure);
-			g_free (data);
+			free_load_source_data (data);
 			return;
 		}
 		else {
@@ -717,8 +731,7 @@ load_source_auth_cb (EBook *book, EBookStatus status, gpointer closure)
 
 	data->cb (book, status, data->closure);
 
-	g_object_unref (data->source);
-	g_free (data);
+	free_load_source_data (data);
 }
 
 static gboolean
@@ -805,6 +818,11 @@ load_source_cb (EBook *book, EBookStatus status, gpointer closure)
 {
 	LoadSourceData *load_source_data = closure;
 
+	if (load_source_data->cancelled) {
+		free_load_source_data (load_source_data);
+		return;
+	}
+
 	if (status == E_BOOK_ERROR_OK && book != NULL) {
 		const gchar *auth;
 
@@ -821,11 +839,10 @@ load_source_cb (EBook *book, EBookStatus status, gpointer closure)
 	}
 	
 	load_source_data->cb (book, status, load_source_data->closure);
-	g_object_unref (load_source_data->source);
-	g_free (load_source_data);
+	free_load_source_data (load_source_data);
 }
 
-void
+guint
 addressbook_load_source (EBook *book, ESource *source,
 			 EBookCallback cb, gpointer closure)
 {
@@ -834,8 +851,30 @@ addressbook_load_source (EBook *book, ESource *source,
 	load_source_data->cb = cb;
 	load_source_data->closure = closure;
 	load_source_data->source = g_object_ref (source);
+	load_source_data->cancelled = FALSE;
 
 	e_book_async_load_source (book, source, load_source_cb, load_source_data);
+
+	return GPOINTER_TO_UINT (load_source_data);
+}
+
+void
+addressbook_load_source_cancel (guint id)
+{
+	LoadSourceData *load_source_data = GUINT_TO_POINTER (id);
+
+	load_source_data->cancelled = TRUE;
+}
+
+static void
+default_book_cb (EBook *book, EBookStatus status, gpointer closure)
+{
+	LoadSourceData *load_source_data = closure;
+
+	if (status == E_BOOK_ERROR_OK)
+		load_source_data->source = g_object_ref (e_book_get_source (book));
+
+	load_source_cb (book, status, closure);
 }
 
 void
@@ -843,12 +882,11 @@ addressbook_load_default_book (EBookCallback cb, gpointer closure)
 {
 	LoadSourceData *load_source_data = g_new (LoadSourceData, 1);
 
-	/* FIXME: We need to get the source for the default book */
-
 	load_source_data->cb = cb;
+	load_source_data->source = NULL;
 	load_source_data->closure = closure;
 
-	e_book_async_get_default_addressbook (load_source_cb, load_source_data);
+	e_book_async_get_default_addressbook (default_book_cb, load_source_data);
 }
 
 static void
@@ -950,9 +988,11 @@ set_prop (BonoboPropertyBag *bag,
 						       gtk_notebook_page_num (GTK_NOTEBOOK (view->notebook),
 									      uid_view));
 
-			/* change menus/toolbars to reflect the new view */
-			eab_view_setup_menus (EAB_VIEW (uid_view), bonobo_control_get_ui_component (view->control));
-			update_command_state (EAB_VIEW (uid_view), view);
+			/* change menus/toolbars to reflect the new view, assuming we are already displayed */
+			if (bonobo_ui_component_get_container (bonobo_control_get_ui_component (view->control)) != CORBA_OBJECT_NIL) {
+				eab_view_setup_menus (EAB_VIEW (uid_view), bonobo_control_get_ui_component (view->control));
+				update_command_state (EAB_VIEW (uid_view), view);
+			}
 		}
 		else {
 			g_warning ("Could not find source by UID '%s'!", uid);
