@@ -491,7 +491,7 @@ e_select_names_init (ESelectNames *e_select_names)
 
 	gtk_window_set_modal (GTK_WINDOW (e_select_names), TRUE);
 	gtk_window_set_default_size (GTK_WINDOW (e_select_names), 472, 512);
-	gtk_window_set_title (GTK_WINDOW (e_select_names), _("Select Contents from Address Book"));
+	gtk_window_set_title (GTK_WINDOW (e_select_names), _("Select Contacts from Address Book"));
 	gtk_window_set_resizable (GTK_WINDOW (e_select_names), TRUE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (e_select_names), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (e_select_names), 4);
