@@ -183,7 +183,7 @@ static char * countries [] = {
 	N_("Colombia"),
 	N_("Comoros"),
 	N_("Congo"),
-	N_("Congo"),
+	N_("Congo, The Democratic Republic Of The"),
 	N_("Cook Islands"),
 	N_("Costa Rica"),
 	N_("Cote d'Ivoire"),
@@ -195,7 +195,6 @@ static char * countries [] = {
 	N_("Djibouti"),
 	N_("Dominica"),
 	N_("Dominican Republic"),
-	N_("East Timor"),
 	N_("Ecuador"),
 	N_("Egypt"),
 	N_("El Salvador"),
@@ -223,6 +222,7 @@ static char * countries [] = {
 	N_("Guadeloupe"),
 	N_("Guam"),
 	N_("Guatemala"),
+	N_("Guernsey"),
 	N_("Guinea"),
 	N_("Guinea-bissau"),
 	N_("Guyana"),
@@ -235,15 +235,20 @@ static char * countries [] = {
 	N_("Iceland"),
 	N_("India"),
 	N_("Indonesia"),
+	N_("Iran"),
+	N_("Iraq"),
 	N_("Ireland"),
+	N_("Isle of Man"),
 	N_("Israel"),
 	N_("Italy"),
 	N_("Jamaica"),
 	N_("Japan"),
+	N_("Jersey"),
 	N_("Jordan"),
 	N_("Kazakhstan"),
 	N_("Kenya"),
 	N_("Kiribati"),
+	N_("Korea, Democratic People's Republic Of"),
 	N_("Korea, Republic Of"),
 	N_("Kuwait"),
 	N_("Kyrgyzstan"),
@@ -252,10 +257,11 @@ static char * countries [] = {
 	N_("Lebanon"),
 	N_("Lesotho"),
 	N_("Liberia"),
+	N_("Libya"),
 	N_("Liechtenstein"),
 	N_("Lithuania"),
 	N_("Luxembourg"),
-	N_("Macau"),
+	N_("Macao"),
 	N_("Macedonia"),
 	N_("Madagascar"),
 	N_("Malawi"),
@@ -317,6 +323,7 @@ static char * countries [] = {
 	N_("Sao Tome And Principe"),
 	N_("Saudi Arabia"),
 	N_("Senegal"),
+	N_("Serbia And Montenegro"),
 	N_("Seychelles"),
 	N_("Sierra Leone"),
 	N_("Singapore"),
@@ -336,10 +343,12 @@ static char * countries [] = {
 	N_("Swaziland"),
 	N_("Sweden"),
 	N_("Switzerland"),
+	N_("Syria"),
 	N_("Taiwan"),
 	N_("Tajikistan"),
 	N_("Tanzania, United Republic Of"),
 	N_("Thailand"),
+	N_("Timor-Leste"),
 	N_("Togo"),
 	N_("Tokelau"),
 	N_("Tonga"),
@@ -364,7 +373,6 @@ static char * countries [] = {
 	N_("Wallis And Futuna Islands"),
 	N_("Western Sahara"),
 	N_("Yemen"),
-	N_("Yugoslavia"),
 	N_("Zambia"),
 	N_("Zimbabwe"),
 	NULL
