@@ -26,6 +26,7 @@
 
 #include <string.h>
 #include <glib.h>
+#include <gtk/gtkmain.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-init.h>
 #include <libgnome/gnome-sound.h>
@@ -41,7 +42,6 @@
 #include "alarm-queue.h"
 #include "alarm-notify.h"
 #include "config-data.h"
-#include "save.h"
 
 
 
@@ -103,7 +103,9 @@ alarm_notify_factory_fn (BonoboGenericFactory *factory, const char *component_id
 		g_assert (alarm_notify_service != NULL);
 	}
 
-	return NULL;
+	bonobo_object_ref (BONOBO_OBJECT (alarm_notify_service));
+
+	return BONOBO_OBJECT (alarm_notify_service);
 }
 
 /* Loads the calendars that the alarm daemon has been told to load in the past */
