@@ -26,7 +26,6 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkmessagedialog.h>
 #include <libgnome/gnome-i18n.h>
-#include <gal/widgets/e-unicode.h>
 #include "../calendar-config.h"
 #include "delete-comp.h"
 
@@ -55,9 +54,9 @@
  * unconditionally return TRUE.
  **/
 gboolean
-delete_component_dialog (CalComponent *comp,
+delete_component_dialog (ECalComponent *comp,
 			 gboolean consider_as_untitled,
-			 int n_comps, CalComponentVType vtype,
+			 int n_comps, ECalComponentVType vtype,
 			 GtkWidget *widget)
 {
 	char *str;
@@ -65,11 +64,11 @@ delete_component_dialog (CalComponent *comp,
 	int ret;
 
 	if (comp) {
-		g_return_val_if_fail (IS_CAL_COMPONENT (comp), FALSE);
+		g_return_val_if_fail (E_IS_CAL_COMPONENT (comp), FALSE);
 		g_return_val_if_fail (n_comps == 1, FALSE);
 	} else {
 		g_return_val_if_fail (n_comps > 1, FALSE);
-		g_return_val_if_fail (vtype != CAL_COMPONENT_NO_TYPE, FALSE);
+		g_return_val_if_fail (vtype != E_CAL_COMPONENT_NO_TYPE, FALSE);
 	}
 
 	g_return_val_if_fail (widget != NULL, FALSE);
@@ -79,19 +78,19 @@ delete_component_dialog (CalComponent *comp,
 		return TRUE;
 
 	if (comp) {
-		CalComponentText summary;
+		ECalComponentText summary;
 		char *tmp;
 
-		vtype = cal_component_get_vtype (comp);
+		vtype = e_cal_component_get_vtype (comp);
 
 		if (!consider_as_untitled) {
-			cal_component_get_summary (comp, &summary);
+			e_cal_component_get_summary (comp, &summary);
 			tmp = g_strdup (summary.value);
 		} else
 			tmp = NULL;
 
 		switch (vtype) {
-		case CAL_COMPONENT_EVENT:
+		case E_CAL_COMPONENT_EVENT:
 			if (tmp)
 				str = g_strdup_printf (_("Are you sure you want to delete "
 							 "the appointment `%s'?"), tmp);
@@ -100,7 +99,7 @@ delete_component_dialog (CalComponent *comp,
 						  "untitled appointment?"));
 			break;
 
-		case CAL_COMPONENT_TODO:
+		case E_CAL_COMPONENT_TODO:
 			if (tmp)
 				str = g_strdup_printf (_("Are you sure you want to delete "
 							 "the task `%s'?"), tmp);
@@ -109,7 +108,7 @@ delete_component_dialog (CalComponent *comp,
 						  "untitled task?"));
 			break;
 
-		case CAL_COMPONENT_JOURNAL:
+		case E_CAL_COMPONENT_JOURNAL:
 			if (tmp)
 				str = g_strdup_printf (_("Are you sure you want to delete "
 							 "the journal entry `%s'?"), tmp);
@@ -128,17 +127,17 @@ delete_component_dialog (CalComponent *comp,
 		g_free (tmp);
 	} else {
 		switch (vtype) {
-		case CAL_COMPONENT_EVENT:
+		case E_CAL_COMPONENT_EVENT:
 			str = g_strdup_printf (_("Are you sure you want to delete "
 						 "%d appointments?"), n_comps);
 			break;
 
-		case CAL_COMPONENT_TODO:
+		case E_CAL_COMPONENT_TODO:
 			str = g_strdup_printf (_("Are you sure you want to delete "
 						 "%d tasks?"), n_comps);
 			break;
 
-		case CAL_COMPONENT_JOURNAL:
+		case E_CAL_COMPONENT_JOURNAL:
 			str = g_strdup_printf (_("Are you sure you want to delete "
 						 "%d journal entries?"), n_comps);
 			break;
@@ -150,7 +149,7 @@ delete_component_dialog (CalComponent *comp,
 		}
 	}
 
-	dialog = gtk_message_dialog_new (gtk_widget_get_toplevel (widget),
+	dialog = gtk_message_dialog_new ((GtkWindow *)gtk_widget_get_toplevel (widget),
 					 0, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "%s", str);
 	g_free (str);
 	ret = gtk_dialog_run ((GtkDialog *)dialog) == GTK_RESPONSE_YES;
