@@ -195,7 +195,7 @@ ensure_task_not_complete (ECalModelComponent *comp_data)
 	}
 
 	/* Status. */
-	prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_PERCENTCOMPLETE_PROPERTY);
+	prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_STATUS_PROPERTY);
 	if (prop)
 		icalproperty_set_status (prop, ICAL_STATUS_NEEDSACTION);
 }
@@ -324,8 +324,7 @@ get_status (ECalModelComponent *comp_data)
 		case ICAL_STATUS_CANCELLED:
 			return _("Cancelled");
 		default:
-			g_assert_not_reached ();
-			return NULL;
+			return "";
 		}
 	}
 
@@ -960,8 +959,7 @@ ecmt_get_color_for_component (ECalModel *model, ECalModelComponent *comp_data)
 		break;
 	}
 
-	return "black";
-	/* return E_CAL_MODEL_CLASS (parent_class)->get_color_for_component (model, comp_data); */
+	return E_CAL_MODEL_CLASS (parent_class)->get_color_for_component (model, comp_data);
 }
 
 static void
