@@ -321,7 +321,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	g_object_set (G_OBJECT (cell),
 		      "strikeout_column", E_CAL_MODEL_TASKS_FIELD_COMPLETE,
 		      "bold_column", E_CAL_MODEL_TASKS_FIELD_OVERDUE,
-		      "color_column", E_CAL_MODEL_FIELD_COLOR,
+		      "bg_color_column", E_CAL_MODEL_FIELD_COLOR,
 		      NULL);
 
 	e_table_extras_add_cell (extras, "calstring", cell);
@@ -334,7 +334,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	g_object_set (G_OBJECT (cell),
 		      "strikeout_column", E_CAL_MODEL_TASKS_FIELD_COMPLETE,
 		      "bold_column", E_CAL_MODEL_TASKS_FIELD_OVERDUE,
-		      "color_column", E_CAL_MODEL_FIELD_COLOR,
+		      "bg_color_column", E_CAL_MODEL_FIELD_COLOR,
 		      NULL);
 
 	popup_cell = e_cell_date_edit_new ();
@@ -357,7 +357,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	g_object_set (G_OBJECT (cell),
 		      "strikeout_column", E_CAL_MODEL_TASKS_FIELD_COMPLETE,
 		      "bold_column", E_CAL_MODEL_TASKS_FIELD_OVERDUE,
-		      "color_column", E_CAL_MODEL_FIELD_COLOR,
+		      "bg_color_column", E_CAL_MODEL_FIELD_COLOR,
 		      "editable", FALSE,
 		      NULL);
 
@@ -379,7 +379,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	g_object_set (G_OBJECT (cell),
 		      "strikeout_column", E_CAL_MODEL_TASKS_FIELD_COMPLETE,
 		      "bold_column", E_CAL_MODEL_TASKS_FIELD_OVERDUE,
-		      "color_column", E_CAL_MODEL_FIELD_COLOR,
+		      "bg_color_column", E_CAL_MODEL_FIELD_COLOR,
 		      "editable", FALSE,
 		      NULL);
 
@@ -402,7 +402,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	g_object_set (G_OBJECT (cell),
 		      "strikeout_column", E_CAL_MODEL_TASKS_FIELD_COMPLETE,
 		      "bold_column", E_CAL_MODEL_TASKS_FIELD_OVERDUE,
-		      "color_column", E_CAL_MODEL_FIELD_COLOR,
+		      "bg_color_column", E_CAL_MODEL_FIELD_COLOR,
 		      NULL);
 
 	popup_cell = e_cell_combo_new ();
@@ -431,7 +431,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	g_object_set (G_OBJECT (cell),
 		      "strikeout_column", E_CAL_MODEL_TASKS_FIELD_COMPLETE,
 		      "bold_column", E_CAL_MODEL_TASKS_FIELD_OVERDUE,
-		      "color_column", E_CAL_MODEL_FIELD_COLOR,
+		      "bg_color_column", E_CAL_MODEL_FIELD_COLOR,
 		      "editable", FALSE,
 		      NULL);
 
@@ -452,7 +452,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	g_object_set (G_OBJECT (cell),
 		      "strikeout_column", E_CAL_MODEL_TASKS_FIELD_COMPLETE,
 		      "bold_column", E_CAL_MODEL_TASKS_FIELD_OVERDUE,
-		      "color_column", E_CAL_MODEL_FIELD_COLOR,
+		      "bg_color_column", E_CAL_MODEL_FIELD_COLOR,
 		      "editable", FALSE,
 		      NULL);
 
@@ -724,7 +724,7 @@ e_calendar_table_delete_selected (ECalendarTable *cal_table)
 	ETable *etable;
 	int n_selected;
 	ECalModelComponent *comp_data;
-	ECalComponent *comp;
+	ECalComponent *comp = NULL;
 
 	g_return_if_fail (cal_table != NULL);
 	g_return_if_fail (E_IS_CALENDAR_TABLE (cal_table));
@@ -742,16 +742,18 @@ e_calendar_table_delete_selected (ECalendarTable *cal_table)
 
 	/* FIXME: this may be something other than a TODO component */
 
-	comp = e_cal_component_new ();
-	if (comp_data)
+	if (comp_data) {
+		comp = e_cal_component_new ();
 		e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (comp_data->icalcomp));
-
+	}
+	
 	if (delete_component_dialog (comp, FALSE, n_selected, E_CAL_COMPONENT_TODO,
 				     GTK_WIDGET (cal_table)))
 		delete_selected_components (cal_table);
 
 	/* free memory */
-	g_object_unref (comp);
+	if (comp)
+		g_object_unref (comp);
 }
 
 /**
@@ -1098,21 +1100,20 @@ e_calendar_table_show_popup_menu (ETable *table,
 	int disable_mask = 0;
 	GtkMenu *gtk_menu;
 	icalproperty *prop;
+	GSList *selection;
 	ECalModelComponent *comp_data;
 	gboolean read_only = TRUE;
-	
-	n_selected = e_table_selected_count (table);
-	if (n_selected <= 0)
+
+	selection = get_selected_objects (cal_table);
+	if (!selection)
 		return TRUE;
 
-	comp_data = get_selected_comp (cal_table);
-	g_assert (comp_data != NULL);
-
+	comp_data = selection->data;
+	n_selected = g_slist_length (selection);
 	if (n_selected == 1) {
 		hide_mask = MASK_MULTIPLE;
 
 		/* See if the task has the URL property set */
-
 		prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_URL_PROPERTY);
 		if (!prop)
 			disable_mask |= MASK_LACKS_URL;
@@ -1132,6 +1133,8 @@ e_calendar_table_show_popup_menu (ETable *table,
                                                                             
         e_popup_menu (gtk_menu, gdk_event);
 
+	g_slist_free (selection);
+
 	return TRUE;
 }
 
@@ -1376,7 +1379,7 @@ static GdkPixbuf *progress_icon = NULL;
 void
 e_calendar_table_set_status_message (ECalendarTable *cal_table, const gchar *message)
 {
-	EActivityHandler *activity_handler = calendar_component_peek_activity_handler (calendar_component_peek ());
+	EActivityHandler *activity_handler = tasks_component_peek_activity_handler (tasks_component_peek ());
 
         g_return_if_fail (E_IS_CALENDAR_TABLE (cal_table));
 
