@@ -1015,7 +1015,8 @@ setup_widgets (GnomeCalendar *gcal)
 	connect_week_view_focus (gcal, E_WEEK_VIEW (priv->month_view));
 
 	/* The List View. */
-	filename = g_strdup_printf (".evolution/config/CalListView");
+	filename = g_build_filename (calendar_component_peek_config_directory (calendar_component_peek ()),
+				     "CalListView", NULL);
 	priv->list_view = e_cal_list_view_new (filename);
 	g_free (filename);
 
@@ -1606,7 +1607,8 @@ gnome_calendar_setup_view_menus (GnomeCalendar *gcal, BonoboUIComponent *uic)
 
 		gal_view_collection_set_title (collection, _("Calendar"));
 
- 		path = gnome_util_prepend_user_home ("/evolution/views/calendar/");
+		path = g_build_filename (calendar_component_peek_base_directory (calendar_component_peek ()), 
+					 "calendar", "views", NULL);
 		gal_view_collection_set_storage_directories (collection,
 							     EVOLUTION_GALVIEWSDIR "/calendar/",
 							     path);
@@ -1897,6 +1899,7 @@ backend_error_cb (ECal *client, const char *message, gpointer data)
 static void
 backend_died_cb (ECal *client, gpointer data)
 {
+	GtkWidget *dialog;
 	GnomeCalendar *gcal;
 	GnomeCalendarPrivate *priv;
 	char *message;
@@ -1905,27 +1908,35 @@ backend_died_cb (ECal *client, gpointer data)
 	gcal = GNOME_CALENDAR (data);
 	priv = gcal->priv;
 
-	/* FIXME This doesn't remove the calendar from the list or anything */
 	uristr = get_uri_without_password (e_cal_get_uri (client));
 	if (client == priv->task_pad_client) {
-		message = g_strdup_printf (_("The task backend for\n%s\n has crashed. "
-					     "You will have to restart Evolution in order "
-					     "to use it again"),
-					   uristr);
+		message = g_strdup_printf (_("The task backend for\n%s\n has crashed."), uristr);
 		e_calendar_table_set_status_message (E_CALENDAR_TABLE (priv->todo), NULL);
+
+		g_object_unref (priv->task_pad_client);
+		priv->task_pad_client = NULL;
 	} else {
 		int i;
 		
-		message = g_strdup_printf (_("The calendar backend for\n%s\n has crashed. "
-					     "You will have to restart Evolution in order "
-					     "to use it again"),
-					   uristr);
+		message = g_strdup_printf (_("The calendar backend for\n%s\n has crashed."), uristr);
 		
 		for (i = 0; i < GNOME_CAL_LAST_VIEW; i++)
 			e_calendar_view_set_status_message (priv->views[i], NULL);
+
+		e_source_selector_unselect_source (
+			calendar_component_peek_source_selector (calendar_component_peek ()),
+			e_cal_get_source (client));
+
+		g_hash_table_remove (priv->clients, e_cal_get_uri (client));
+		priv->clients_list = g_list_remove (priv->clients_list, client);
+		g_object_unref (client);
 	}
 
-	gnome_error_dialog_parented (message, GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (gcal))));
+	dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (gcal))),
+					 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+					 message);
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 	g_free (message);
 	g_free (uristr);
 }
@@ -1948,34 +1959,33 @@ gnome_calendar_construct (GnomeCalendar *gcal)
 	uid = calendar_config_get_primary_tasks ();
 	if (uid) {
 		ESourceList *source_list;
+		ESource *tasks_source;
 		GConfClient *conf_client;
 
 		conf_client = gconf_client_get_default ();
 		source_list = e_source_list_new_for_gconf (conf_client, "/apps/evolution/tasks/sources");
 
-		priv->task_pad_client = auth_new_cal_from_source (
-			e_source_list_peek_source_by_uid (source_list, uid), E_CAL_SOURCE_TYPE_TODO);
+		tasks_source = e_source_list_peek_source_by_uid (source_list, uid);
+		if (tasks_source) {
+			priv->task_pad_client = auth_new_cal_from_source (tasks_source, E_CAL_SOURCE_TYPE_TODO);
+
+			g_signal_connect (priv->task_pad_client, "backend_error",
+					  G_CALLBACK (backend_error_cb), gcal);
+			g_signal_connect (priv->task_pad_client, "categories_changed",
+					  G_CALLBACK (client_categories_changed_cb), gcal);
+			g_signal_connect (priv->task_pad_client, "backend_died",
+					  G_CALLBACK (backend_died_cb), gcal);
+
+			if (open_ecal (gcal, priv->task_pad_client, TRUE)) {
+				e_cal_model_add_client (e_calendar_table_get_model (E_CALENDAR_TABLE (priv->todo)),
+							priv->task_pad_client);
+			}
+		} else
+			priv->task_pad_client = NULL;
 
 		g_object_unref (conf_client);
 		g_object_unref (source_list);
-
-		if (!priv->task_pad_client)
-			return NULL;
-
-		g_signal_connect (priv->task_pad_client, "backend_error",
-				  G_CALLBACK (backend_error_cb), gcal);
-		g_signal_connect (priv->task_pad_client, "categories_changed",
-				  G_CALLBACK (client_categories_changed_cb), gcal);
-		g_signal_connect (priv->task_pad_client, "backend_died",
-				  G_CALLBACK (backend_died_cb), gcal);
-
 		g_free (uid);
-
-		if (!open_ecal (gcal, priv->task_pad_client, TRUE))
-			return NULL;
-
-		e_cal_model_add_client (e_calendar_table_get_model (E_CALENDAR_TABLE (priv->todo)),
-					priv->task_pad_client);
 	}
 
 	/* Get the default view to show. */
