@@ -47,6 +47,7 @@
 #include <libecal/e-cal-time-util.h>
 #include "calendar-commands.h"
 #include "calendar-config.h"
+#include "e-cal-model.h"
 #include "e-day-view.h"
 #include "e-day-view-layout.h"
 #include "e-week-view.h"
@@ -303,20 +304,20 @@ enum align_box {
 /* Prints a rectangle, with or without a border, filled or outline, and
    possibly with triangular arrows at one or both horizontal edges.
    width      = width of border, -ve means no border.
-   fillcolor = shade of fill,   -ve means no fill.
+   red,green,blue = bgcolor to fill,   -ve means no fill.
    left_triangle_width, right_triangle_width = width from edge of rectangle to
           point of triangle, or -ve for no triangle. */
 static void
 print_border_with_triangles (GnomePrintContext *pc,
 			     double l, double r, double t, double b,
-			     double width, double fillcolor,
+			     double width, double red, double green, double blue,
 			     double left_triangle_width,
 			     double right_triangle_width)
 {
 	gnome_print_gsave (pc);
 
 	/* Fill in the interior of the rectangle, if desired. */
-	if (fillcolor >= -EPSILON) {
+	if (red >= -EPSILON && green >= -EPSILON && blue >= -EPSILON) {
 		gnome_print_moveto (pc, l, t);
 		if (left_triangle_width > 0.0)
 			gnome_print_lineto (pc, l - left_triangle_width,
@@ -328,8 +329,7 @@ print_border_with_triangles (GnomePrintContext *pc,
 					    (t + b) / 2);
 		gnome_print_lineto (pc, r, t);
 		gnome_print_closepath (pc);
-		gnome_print_setrgbcolor (pc, fillcolor, fillcolor,
-					 fillcolor);
+		gnome_print_setrgbcolor (pc, red, green, blue);
 		gnome_print_fill (pc);
 	}
 
@@ -358,15 +358,39 @@ print_border_with_triangles (GnomePrintContext *pc,
 /* Prints a rectangle, with or without a border, and filled or outline.
    width      = width of border, -ve means no border.
    fillcolor = shade of fill,   -ve means no fill. */
+static void
+print_border_rgb (GnomePrintContext *pc,
+	      double l, double r, double t, double b,
+	      double width, double red, double green, double blue)
+{
+	print_border_with_triangles (pc, l, r, t, b, width, red, green, blue, -1.0, -1.0);
+}
+
 static void
 print_border (GnomePrintContext *pc,
 	      double l, double r, double t, double b,
 	      double width, double fillcolor)
 {
-	print_border_with_triangles (pc, l, r, t, b, width, fillcolor,
-				     -1.0, -1.0);
+	print_border_rgb (pc, l, r, t, b, width, fillcolor, fillcolor, fillcolor);
 }
 
+static void
+print_rectangle (GnomePrintContext *pc,
+		 double l, double r, double t, double b,
+		 double red, double green, double blue)
+{
+	gnome_print_gsave (pc);
+
+	gnome_print_moveto (pc, l, t);
+	gnome_print_lineto (pc, l, b);
+	gnome_print_lineto (pc, r, b);
+	gnome_print_lineto (pc, r, t);
+	gnome_print_closepath (pc);
+	gnome_print_setrgbcolor (pc, red, green, blue);
+	gnome_print_fill (pc);
+
+	gnome_print_grestore (pc);
+}
 
 /* Prints 1 line of aligned text in a box. It is centered vertically, and
    the horizontal alignment can be either ALIGN_LEFT, ALIGN_RIGHT, or
@@ -524,7 +548,8 @@ format_date(time_t time, int flags, char *buffer, int bufflen)
 static gboolean 
 instance_cb (ECalComponent *comp, time_t instance_start, time_t instance_end, gpointer data)
 {
-	gboolean *found = data;
+	
+	gboolean *found = ((ECalModelGenerateInstancesData *) data)->cb_data;
 	
 	*found = TRUE;
 	
@@ -639,9 +664,9 @@ print_month_small (GnomePrintContext *pc, GnomeCalendar *gcal, time_t month,
 				sprintf (buf, "%d", day);
 
 				/* this is a slow messy way to do this ... but easy ... */
-				e_cal_generate_instances (client, now, 
-							  time_day_end_with_zone (now, zone),
-							  instance_cb, &found);
+				e_cal_model_generate_instances (gnome_calendar_get_calendar_model (gcal), now, 
+								time_day_end_with_zone (now, zone),
+								instance_cb, &found);
 				
 				font = found ? font_bold : font_normal;
 
@@ -855,7 +880,7 @@ print_day_background (GnomePrintContext *pc, GnomeCalendar *gcal,
 
 /* This adds one event to the view, adding it to the appropriate array. */
 static gint
-print_day_add_event (ECalComponent *comp,
+print_day_add_event (ECalModelComponent *comp_data,
 		     time_t	    start,
 		     time_t	    end,
 		     gint	    days_shown,
@@ -864,68 +889,67 @@ print_day_add_event (ECalComponent *comp,
 		     GArray	  **events)
 
 {
-/* 	icaltimezone *zone = get_timezone (); */
-/* 	EDayViewEvent event; */
-/* 	gint day, offset; */
-/* 	struct icaltimetype start_tt, end_tt; */
+	icaltimezone *zone = get_timezone ();
+	EDayViewEvent event;
+	gint day, offset;
+	struct icaltimetype start_tt, end_tt;
 
-/* #if 0 */
-/* 	g_print ("Day view lower: %s", ctime (&day_starts[0])); */
-/* 	g_print ("Day view upper: %s", ctime (&day_starts[days_shown])); */
-/* 	g_print ("Event start: %s", ctime (&start)); */
-/* 	g_print ("Event end  : %s\n", ctime (&end)); */
-/* #endif */
+#if 0
+	g_print ("Day view lower: %s", ctime (&day_starts[0]));
+	g_print ("Day view upper: %s", ctime (&day_starts[days_shown]));
+	g_print ("Event start: %s", ctime (&start));
+	g_print ("Event end  : %s\n", ctime (&end));
+#endif
 
-/* 	/\* Check that the event times are valid. *\/ */
-/* 	g_return_val_if_fail (start <= end, -1); */
-/* 	g_return_val_if_fail (start < day_starts[days_shown], -1); */
-/* 	g_return_val_if_fail (end > day_starts[0], -1); */
+	/* Check that the event times are valid. */
+	g_return_val_if_fail (start <= end, -1);
+	g_return_val_if_fail (start < day_starts[days_shown], -1);
+	g_return_val_if_fail (end > day_starts[0], -1);
 
-/* 	start_tt = icaltime_from_timet_with_zone (start, FALSE, zone); */
-/* 	end_tt = icaltime_from_timet_with_zone (end, FALSE, zone); */
+	start_tt = icaltime_from_timet_with_zone (start, FALSE, zone);
+	end_tt = icaltime_from_timet_with_zone (end, FALSE, zone);
 
-/* 	event.comp = comp; */
-/* 	g_object_ref (comp); */
-/* 	event.start = start; */
-/* 	event.end = end; */
-/* 	event.canvas_item = NULL; */
+	event.comp_data = comp_data;
+	event.start = start;
+	event.end = end;
+	event.canvas_item = NULL;
 
-/* 	/\* Calculate the start & end minute, relative to the top of the */
-/* 	   display. *\/ */
-/* 	/\*offset = day_view->first_hour_shown * 60 */
-/* 	  + day_view->first_minute_shown;*\/ */
-/* 	offset = 0; */
-/* 	event.start_minute = start_tt.hour * 60 + start_tt.minute - offset; */
-/* 	event.end_minute = end_tt.hour * 60 + end_tt.minute - offset; */
+	/* Calculate the start & end minute, relative to the top of the
+	   display. */
+	/*offset = day_view->first_hour_shown * 60
+	  + day_view->first_minute_shown;*/
+	offset = 0;
+	event.start_minute = start_tt.hour * 60 + start_tt.minute - offset;
+	event.end_minute = end_tt.hour * 60 + end_tt.minute - offset;
 
-/* 	event.start_row_or_col = 0; */
-/* 	event.num_columns = 0; */
+	event.start_row_or_col = 0;
+	event.num_columns = 0;
 
-/* 	/\* Find out which array to add the event to. *\/ */
-/* 	for (day = 0; day < days_shown; day++) { */
-/* 		if (start >= day_starts[day] && end <= day_starts[day + 1]) { */
+	/* Find out which array to add the event to. */
+	for (day = 0; day < days_shown; day++) {
+		if (start >= day_starts[day] && end <= day_starts[day + 1]) {
 
-/* 			/\* Special case for when the appointment ends at */
-/* 			   midnight, i.e. the start of the next day. *\/ */
-/* 			if (end == day_starts[day + 1]) { */
+			/* Special case for when the appointment ends at
+			   midnight, i.e. the start of the next day. */
+			if (end == day_starts[day + 1]) {
 
-/* 				/\* If the event last the entire day, then we */
-/* 				   skip it here so it gets added to the top */
-/* 				   canvas. *\/ */
-/* 				if (start == day_starts[day]) */
-/* 				    break; */
+				/* If the event last the entire day, then we
+				   skip it here so it gets added to the top
+				   canvas. */
+				if (start == day_starts[day])
+				    break;
 
-/* 				event.end_minute = 24 * 60; */
-/* 			} */
+				event.end_minute = 24 * 60;
+			}
 
-/* 			g_array_append_val (events[day], event); */
-/* 			return day; */
-/* 		} */
-/* 	} */
+			g_array_append_val (events[day], event);
+			return day;
+		}
+	}
 
-/* 	/\* The event wasn't within one day so it must be a long event, */
-/* 	   i.e. shown in the top canvas. *\/ */
-/* 	g_array_append_val (long_events, event); */
+	/* The event wasn't within one day so it must be a long event,
+	   i.e. shown in the top canvas. */
+	g_array_append_val (long_events, event);
 	return E_DAY_VIEW_LONG_EVENT;
 }
 
@@ -934,9 +958,10 @@ static gboolean
 print_day_details_cb (ECalComponent *comp, time_t istart, time_t iend,
 		      gpointer data)
 {
-	struct pdinfo *pdi = (struct pdinfo *)data;
+	ECalModelGenerateInstancesData *mdata = (ECalModelGenerateInstancesData *) data;
+	struct pdinfo *pdi = (struct pdinfo *) mdata->cb_data;
 
-	print_day_add_event (comp, istart, iend,
+	print_day_add_event (mdata->comp_data, istart, iend,
 			     pdi->days_shown, pdi->day_starts,
 			     pdi->long_events, pdi->events);
 
@@ -964,7 +989,7 @@ static void
 print_day_long_event (GnomePrintContext *pc, GnomeFont *font,
 		      double left, double right, double top, double bottom,
 		      double row_height, EDayViewEvent *event,
-		      struct pdinfo *pdi)
+		      struct pdinfo *pdi, ECalModel *model)
 {
 	const gchar *summary;
 	double x1, x2, y1, y2;
@@ -972,6 +997,7 @@ print_day_long_event (GnomePrintContext *pc, GnomeFont *font,
 	char *text;
 	char buffer[32];
 	struct tm date_tm;
+	double red, green, blue;
 
 	/* If the event starts before the first day being printed, draw a
 	   triangle. (Note that I am assuming we are just showing 1 day at
@@ -988,7 +1014,9 @@ print_day_long_event (GnomePrintContext *pc, GnomeFont *font,
 	x2 = right - 10;
 	y1 = top - event->start_row_or_col * row_height - 4;
 	y2 = y1 - row_height + 4;
-	print_border_with_triangles (pc, x1, x2, y1, y2, 0.5, 0.95,
+	red = green = blue = 0.95;
+	e_cal_model_get_rgb_color_for_component (model, event->comp_data, &red, &green, &blue);
+	print_border_with_triangles (pc, x1, x2, y1, y2, 0.5, red, green, blue,
 				     left_triangle_width,
 				     right_triangle_width);
 
@@ -1043,7 +1071,7 @@ print_day_long_event (GnomePrintContext *pc, GnomeFont *font,
 static void
 print_day_event (GnomePrintContext *pc, GnomeFont *font,
 		 double left, double right, double top, double bottom,
-		 EDayViewEvent *event, struct pdinfo *pdi)
+		 EDayViewEvent *event, struct pdinfo *pdi, ECalModel *model)
 {
 	const gchar *summary;
 	double x1, x2, y1, y2, col_width, row_height;
@@ -1051,6 +1079,7 @@ print_day_event (GnomePrintContext *pc, GnomeFont *font,
 	char *text, start_buffer[32], end_buffer[32];
 	gboolean display_times = FALSE, free_text = FALSE;
 	struct tm date_tm;
+	double red, green, blue;
 
 	if ((event->start_minute >= pdi->end_minute_offset)
 	    || (event->end_minute <= pdi->start_minute_offset))
@@ -1081,7 +1110,9 @@ print_day_event (GnomePrintContext *pc, GnomeFont *font,
 		 x1, y1, x2, y2, row_height, start_row, top, pdi->rows);
 #endif
 
-	print_border (pc, x1, x2, y1, y2, 1.0, 0.95);
+	red = green = blue = 0.95;
+	e_cal_model_get_rgb_color_for_component (model, event->comp_data, &red, &green, &blue);
+	print_border_rgb (pc, x1, x2, y1, y2, 1.0, red, green, blue);
 
 	summary = icalcomponent_get_summary (event->comp_data->icalcomp);
 	text = summary ? (char*) summary : "";
@@ -1123,13 +1154,13 @@ print_day_details (GnomePrintContext *pc, GnomeCalendar *gcal, time_t whence,
 		   double left, double right, double top, double bottom)
 {
 	icaltimezone *zone = get_timezone ();
-	ECal *client;
 	EDayViewEvent *event;
 	GnomeFont *font;
 	time_t start, end;
 	struct pdinfo pdi;
 	gint rows_in_top_display, i;
 	double font_size, max_font_size;
+	ECalModel *model = gnome_calendar_get_calendar_model (gcal);
 
 	start = time_day_begin_with_zone (whence, zone);
 	end = time_day_end_with_zone (start, zone);
@@ -1150,8 +1181,7 @@ print_day_details (GnomePrintContext *pc, GnomeCalendar *gcal, time_t whence,
 	pdi.use_24_hour_format = calendar_config_get_24_hour_format ();
 
 	/* Get the events from the server. */
-	client = gnome_calendar_get_default_client (gcal);
-	e_cal_generate_instances (client, start, end, print_day_details_cb, &pdi);
+	e_cal_model_generate_instances (model, start, end, print_day_details_cb, &pdi);
 	qsort (pdi.long_events->data, pdi.long_events->len,
 	       sizeof (EDayViewEvent), e_day_view_event_sort_func);
 	qsort (pdi.events[0]->data, pdi.events[0]->len,
@@ -1189,7 +1219,7 @@ print_day_details (GnomePrintContext *pc, GnomeCalendar *gcal, time_t whence,
 	for (i = 0; i < pdi.long_events->len; i++) {
 		event = &g_array_index (pdi.long_events, EDayViewEvent, i);
 		print_day_long_event (pc, font, left, right, top, bottom,
-				      DAY_VIEW_ROW_HEIGHT, event, &pdi);
+				      DAY_VIEW_ROW_HEIGHT, event, &pdi, model);
 	}
 	g_object_unref (font);
 
@@ -1226,7 +1256,7 @@ print_day_details (GnomePrintContext *pc, GnomeCalendar *gcal, time_t whence,
 	for (i = 0; i < pdi.events[0]->len; i++) {
 		event = &g_array_index (pdi.events[0], EDayViewEvent, i);
 		print_day_event (pc, font, left, right, top, bottom,
-				 event, &pdi);
+				 event, &pdi, model);
 	}
 	g_object_unref (font);
 
@@ -1237,54 +1267,6 @@ print_day_details (GnomePrintContext *pc, GnomeCalendar *gcal, time_t whence,
 	g_array_free (pdi.events[0], TRUE);
 }
 
-
-/* This adds one event to the view, adding it to the appropriate array. */
-static gboolean
-print_week_summary_cb (ECalComponent *comp,
-		       time_t	  start,
-		       time_t	  end,
-		       gpointer	  data)
-
-{
-/* 	icaltimezone *zone = get_timezone (); */
-/* 	EWeekViewEvent event; */
-/* 	struct icaltimetype start_tt, end_tt; */
-
-/* 	struct psinfo *psi = (struct psinfo *)data; */
-
-/* 	/\* Check that the event times are valid. *\/ */
-
-/* #if 0 */
-/* 	g_print ("View start:%li end:%li  Event start:%li end:%li\n", */
-/* 		 psi->day_starts[0], psi->day_starts[psi->days_shown], */
-/* 		 start, end); */
-/* #endif */
-
-/* 	g_return_val_if_fail (start <= end, TRUE); */
-/* 	g_return_val_if_fail (start < psi->day_starts[psi->days_shown], TRUE); */
-/* 	g_return_val_if_fail (end > psi->day_starts[0], TRUE); */
-
-/* 	start_tt = icaltime_from_timet_with_zone (start, FALSE, zone); */
-/* 	end_tt = icaltime_from_timet_with_zone (end, FALSE, zone); */
-
-/* 	event.comp = comp; */
-/* 	g_object_ref (event.comp); */
-/* 	event.start = start; */
-/* 	event.end = end; */
-/* 	event.spans_index = 0; */
-/* 	event.num_spans = 0; */
-
-/* 	event.start_minute = start_tt.hour * 60 + start_tt.minute; */
-/* 	event.end_minute = end_tt.hour * 60 + end_tt.minute; */
-/* 	if (event.end_minute == 0 && start != end) */
-/* 		event.end_minute = 24 * 60; */
-
-/* 	g_array_append_val (psi->events, event); */
-
-	return TRUE;
-}
-
-
 /* Returns TRUE if the event is a one-day event (i.e. not a long event). */
 static gboolean
 print_is_one_day_week_event (EWeekViewEvent *event,
@@ -1309,7 +1291,7 @@ print_week_long_event (GnomePrintContext *pc, GnomeFont *font,
 		       struct psinfo *psi,
 		       double x1, double x2, double y1, double y2,
 		       EWeekViewEvent *event, EWeekViewEventSpan *span,
-		       char *text)
+		       char *text, double red, double green, double blue)
 {
 	double left_triangle_width = -1.0, right_triangle_width = -1.0;
 	struct tm date_tm;
@@ -1325,7 +1307,7 @@ print_week_long_event (GnomePrintContext *pc, GnomeFont *font,
 	if (event->end > psi->day_starts[span->start_day + span->num_days])
 		right_triangle_width = 4;
 
-	print_border_with_triangles (pc, x1, x2, y1, y2, 0.5, 0.9,
+	print_border_with_triangles (pc, x1, x2, y1, y2, 0.5, red, green, blue,
 				     left_triangle_width,
 				     right_triangle_width);
 
@@ -1378,7 +1360,7 @@ print_week_day_event (GnomePrintContext *pc, GnomeFont *font,
 		      struct psinfo *psi,
 		      double x1, double x2, double y1, double y2,
 		      EWeekViewEvent *event, EWeekViewEventSpan *span,
-		      char *text)
+		      char *text, double red, double green, double blue)
 {
 	struct tm date_tm;
 	char buffer[32];
@@ -1394,10 +1376,9 @@ print_week_day_event (GnomePrintContext *pc, GnomeFont *font,
 	e_time_format_time (&date_tm, psi->use_24_hour_format, FALSE,
 			    buffer, sizeof (buffer));
 
+	print_rectangle (pc, x1, x2, y1, y2, red, green, blue);
 	print_text_size (pc, buffer, ALIGN_LEFT, x1, x2, y1, y2);
-	x1 += gnome_font_get_width_utf8 (font, buffer);
-
-	x1 += 4;
+	x1 += gnome_font_get_width_utf8 (font, buffer) + 4;
 	print_text_size (pc, text, ALIGN_LEFT, x1, x2, y1, y2);
 }
 
@@ -1407,6 +1388,7 @@ print_week_event (GnomePrintContext *pc, GnomeFont *font,
 		  struct psinfo *psi,
 		  double left, double top,
 		  double cell_width, double cell_height,
+		  ECalModel *model,
 		  EWeekViewEvent *event, GArray *spans)
 {
 	EWeekViewEventSpan *span;
@@ -1415,6 +1397,7 @@ print_week_event (GnomePrintContext *pc, GnomeFont *font,
 	char *text;
 	int num_days, start_x, start_y, start_h, end_x, end_y, end_h;
 	double x1, x2, y1, y2;
+	double red, green, blue;
 
 	summary = icalcomponent_get_summary (event->comp_data->icalcomp);
 	text = summary ? (char*) summary : "";
@@ -1461,15 +1444,19 @@ print_week_event (GnomePrintContext *pc, GnomeFont *font,
 				- span->row * psi->row_height;
 			y2 = y1 - psi->row_height * 0.9;
 
+			red = .9;
+			green = .9;
+			blue = .9;
+			e_cal_model_get_rgb_color_for_component (model, event->comp_data, &red, &green, &blue);
 			if (print_is_one_day_week_event (event, span,
 							 psi->day_starts)) {
 				print_week_day_event (pc, font, psi,
 						      x1, x2, y1, y2,
-						      event, span, text);
+						      event, span, text, red, green, blue);
 			} else {
 				print_week_long_event (pc, font, psi,
 						       x1, x2, y1, y2,
-						       event, span, text);
+						       event, span, text, red, green, blue);
 			}
 		}
 	}
@@ -1539,7 +1526,52 @@ print_week_view_background (GnomePrintContext *pc, GnomeFont *font,
 	}
 }
 
+/* This adds one event to the view, adding it to the appropriate array. */
+static gboolean
+print_week_summary_cb (ECalComponent *comp,
+		       time_t	  start,
+		       time_t	  end,
+		       gpointer	  data)
 
+{
+ 	icaltimezone *zone = get_timezone ();
+ 	EWeekViewEvent event;
+ 	struct icaltimetype start_tt, end_tt;
+	ECalModelGenerateInstancesData *mdata = (ECalModelGenerateInstancesData *) data;
+ 	struct psinfo *psi = (struct psinfo *) mdata->cb_data;
+
+ 	/* Check that the event times are valid. */
+
+#if 0
+	g_print ("View start:%li end:%li  Event start:%li end:%li\n",
+		 psi->day_starts[0], psi->day_starts[psi->days_shown],
+		 start, end);
+#endif
+
+ 	g_return_val_if_fail (start <= end, TRUE);
+ 	g_return_val_if_fail (start < psi->day_starts[psi->days_shown], TRUE);
+ 	g_return_val_if_fail (end > psi->day_starts[0], TRUE);
+
+ 	start_tt = icaltime_from_timet_with_zone (start, FALSE, zone);
+ 	end_tt = icaltime_from_timet_with_zone (end, FALSE, zone);
+
+	event.comp_data = mdata->comp_data;
+	event.allocated_comp_data = FALSE;
+
+	event.start = start;
+	event.end = end;
+	event.spans_index = 0;
+	event.num_spans = 0;
+
+	event.start_minute = start_tt.hour * 60 + start_tt.minute;
+	event.end_minute = end_tt.hour * 60 + end_tt.minute;
+	if (event.end_minute == 0 && start != end)
+		event.end_minute = 24 * 60;
+
+	g_array_append_val (psi->events, event);
+
+	return TRUE;
+}
 
 static void
 print_week_summary (GnomePrintContext *pc, GnomeCalendar *gcal,
@@ -1548,7 +1580,6 @@ print_week_summary (GnomePrintContext *pc, GnomeCalendar *gcal,
 		    double left, double right, double top, double bottom)
 {
 	icaltimezone *zone = get_timezone ();
-	ECal *client;
 	EWeekViewEvent *event;
 	struct psinfo psi;
 	time_t day_start;
@@ -1556,6 +1587,7 @@ print_week_summary (GnomePrintContext *pc, GnomeCalendar *gcal,
 	GArray *spans;
 	GnomeFont *font;
 	double cell_width, cell_height;
+	ECalModel *model = gnome_calendar_get_calendar_model (gcal);
 
 	psi.days_shown = weeks_shown * 7;
 	psi.events = g_array_new (FALSE, FALSE, sizeof (EWeekViewEvent));
@@ -1585,11 +1617,9 @@ print_week_summary (GnomePrintContext *pc, GnomeCalendar *gcal,
 	}
 
 	/* Get the events from the server. */
-	client = gnome_calendar_get_default_client (gcal);
-	e_cal_generate_instances (client,
-				  psi.day_starts[0],
-				  psi.day_starts[psi.days_shown],
-				  print_week_summary_cb, &psi);
+	e_cal_model_generate_instances (model,
+					psi.day_starts[0], psi.day_starts[psi.days_shown],
+					print_week_summary_cb, &psi);
 	qsort (psi.events->data, psi.events->len,
 	       sizeof (EWeekViewEvent), e_week_view_event_sort_func);
 
@@ -1631,7 +1661,7 @@ print_week_summary (GnomePrintContext *pc, GnomeCalendar *gcal,
 	for (event_num = 0; event_num < psi.events->len; event_num++) {
 		event = &g_array_index (psi.events, EWeekViewEvent, event_num);
 		print_week_event (pc, font, &psi, left, top,
-				  cell_width, cell_height, event, spans);
+				  cell_width, cell_height, model, event, spans);
 	}
 
 	g_object_unref (font);
@@ -2430,7 +2460,6 @@ print_calendar (GnomeCalendar *gcal, gboolean preview, time_t date,
 	GnomePrintContext *pc;
 	int copies, collate;
 	double l, r, t, b;
-	gchar *old_orientation;
 
 	g_return_if_fail (gcal != NULL);
 	g_return_if_fail (GNOME_IS_CALENDAR (gcal));
@@ -2481,10 +2510,10 @@ print_calendar (GnomeCalendar *gcal, gboolean preview, time_t date,
 		gtk_widget_destroy (gpd);
 	}
 
-	old_orientation = gnome_print_config_get (print_config, GNOME_PRINT_KEY_PAGE_ORIENTATION);
-
 	if (default_view == PRINT_VIEW_MONTH)
 		gnome_print_config_set (print_config, GNOME_PRINT_KEY_PAGE_ORIENTATION, "R90");
+	else
+		gnome_print_config_set (print_config, GNOME_PRINT_KEY_PAGE_ORIENTATION, "R0");
 
 	pc = gnome_print_job_get_context (gpm);
 	gnome_print_config_get_page_size (print_config, &r, &t);
@@ -2533,8 +2562,6 @@ print_calendar (GnomeCalendar *gcal, gboolean preview, time_t date,
 		gnome_print_job_print (gpm);
 	}
 
-	gnome_print_config_set (print_config, GNOME_PRINT_KEY_PAGE_ORIENTATION, old_orientation);
-	g_free (old_orientation);
 	g_object_unref (gpm);
 }
 
@@ -2553,6 +2580,8 @@ print_comp (ECalComponent *comp, ECal *client, gboolean preview)
 	if (!print_config)
 		print_config = gnome_print_config_default ();
 
+	gnome_print_config_set (print_config, GNOME_PRINT_KEY_PAGE_ORIENTATION, "R0");
+
 	copies = 1;
 	collate = FALSE;
 
@@ -2622,6 +2651,99 @@ print_comp (ECalComponent *comp, ECal *client, gboolean preview)
 	g_object_unref (gpm);
 }
 
+static void
+print_title (GnomePrintContext *pc, const char *title,
+	     double page_width, double page_height)
+{
+	GnomeFont *font;
+	double w, x, y;
+
+	font = gnome_font_find_closest ("Sans Bold", 18);
+
+	w = gnome_font_get_width_utf8 (font, title);
+
+	x = (page_width - w) / 2;
+	y = page_height - gnome_font_get_ascender (font);
+
+	gnome_print_moveto (pc, x, y);
+	gnome_print_setfont (pc, font);
+	gnome_print_setrgbcolor (pc, 0, 0, 0);
+	gnome_print_show (pc, title);
+
+	g_object_unref (font);
+}
+
+void
+print_table (ETable *etable, const char *title, gboolean preview)
+{
+	EPrintable *printable;
+	GnomePrintContext *pc;
+	GnomePrintJob *gpm;
+	double l, r, t, b, page_width, page_height, left_margin, bottom_margin;
+
+	if (!print_config)
+		print_config = gnome_print_config_default ();
+
+	gnome_print_config_set (print_config, GNOME_PRINT_KEY_PAGE_ORIENTATION, "R0");
+	printable = e_table_get_printable (etable);
+	g_object_ref (printable);
+	gtk_object_sink (GTK_OBJECT (printable));
+	e_printable_reset (printable);
+
+	gpm = gnome_print_job_new (print_config);
+	pc = gnome_print_job_get_context (gpm);
+
+	gnome_print_config_get_page_size (print_config, &r, &t);
+
+#if 0
+	gnome_print_config_get_double (print_config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP, &temp_d);
+	t -= temp_d;
+	gnome_print_config_get_double (print_config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, &temp_d);
+	r -= temp_d;
+	gnome_print_config_get_double (print_config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, &b);
+	gnome_print_config_get_double (print_config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, &l);
+#endif
+
+	b = t * TEMP_MARGIN;
+	l = r * TEMP_MARGIN;
+	t *= (1.0 - TEMP_MARGIN);
+	r *= (1.0 - TEMP_MARGIN);
+
+	page_width = r - l;
+	page_height = t - b;
+	left_margin = l;
+	bottom_margin = b;
+
+	do {
+		gnome_print_beginpage (pc, "Tasks");
+		gnome_print_gsave (pc);
+
+		gnome_print_translate (pc, left_margin, bottom_margin);
+
+		print_title (pc, title, page_width, page_height);
+
+		if (e_printable_data_left (printable))
+			e_printable_print_page (printable, pc,
+						page_width, page_height - 24, TRUE);
+
+		gnome_print_grestore (pc);
+		gnome_print_showpage (pc);
+	} while (e_printable_data_left (printable));
+
+	gnome_print_job_close (gpm);
+
+	if (preview) {
+		GtkWidget *gpmp;
+		gpmp = gnome_print_job_preview_new (gpm, _("Print Preview"));
+		gtk_widget_show (gpmp);
+	} else {
+		gnome_print_job_print (gpm);
+	}
+
+	g_object_unref (gpm);
+	g_object_unref (printable);
+}
+
 void
 print_setup (void)
 {
