@@ -59,7 +59,9 @@ static BonoboObjectClass *parent_class = NULL;
 extern ECompEditorRegistry *comp_editor_registry;
 
 struct _TasksComponentPrivate {
+	char *base_directory;
 	char *config_directory;
+
 	GConfClient *gconf_client;
 
 	ESourceList *source_list;
@@ -73,28 +75,12 @@ struct _TasksComponentPrivate {
 	ECal *create_ecal;
 
 	GList *notifications;
+
+	EActivityHandler *activity_handler;
 };
 
 /* Utility functions.  */
 /* FIXME Some of these are duplicated from calendar-component.c */
-static void
-add_uri_for_source (ESource *source, ETasks *tasks)
-{
-	char *uri = e_source_get_uri (source);
-
-	e_tasks_add_todo_uri (tasks, uri);
-	g_free (uri);
-}
-
-static void
-remove_uri_for_source (ESource *source, ETasks *tasks)
-{
-	char *uri = e_source_get_uri (source);
-
-	e_tasks_remove_todo_uri (tasks, uri);
-	g_free (uri);
-}
-
 static gboolean
 is_in_selection (GSList *selection, ESource *source)
 {
@@ -125,26 +111,6 @@ is_in_uids (GSList *uids, ESource *source)
 	return FALSE;
 }
 
-static ESource *
-find_first_source (ESourceList *source_list)
-{
-	GSList *groups, *sources, *l, *m;
-			
-	groups = e_source_list_peek_groups (source_list);
-	for (l = groups; l; l = l->next) {
-		ESourceGroup *group = l->data;
-				
-		sources = e_source_group_peek_sources (group);
-		for (m = sources; m; m = m->next) {
-			ESource *source = m->data;
-
-			return source;
-		}				
-	}
-
-	return NULL;
-}
-
 static void
 update_uris_for_selection (TasksComponent *component)
 {
@@ -159,13 +125,13 @@ update_uris_for_selection (TasksComponent *component)
 		ESource *old_selected_source = l->data;
 
 		if (!is_in_selection (selection, old_selected_source))
-			remove_uri_for_source (old_selected_source, E_TASKS (priv->tasks));
+			e_tasks_remove_todo_source (priv->tasks, old_selected_source);
 	}	
 	
 	for (l = selection; l; l = l->next) {
 		ESource *selected_source = l->data;
 		
-		add_uri_for_source (selected_source, E_TASKS (priv->tasks));
+		e_tasks_add_todo_source (priv->tasks, selected_source);
 		uids_selected = g_slist_append (uids_selected, (char *)e_source_peek_uid (selected_source));
 	}
 
@@ -182,7 +148,6 @@ update_uri_for_primary_selection (TasksComponent *component)
 {
 	TasksComponentPrivate *priv;
 	ESource *source;
-	char *uri;
 	ECalendarTable *cal_table;
 	ETable *etable;
 
@@ -193,9 +158,7 @@ update_uri_for_primary_selection (TasksComponent *component)
 		return;
 
 	/* Set the default */
-	uri = e_source_get_uri (source);
-	e_tasks_set_default_uri (priv->tasks, uri);
-	g_free (uri);
+	e_tasks_set_default_source (priv->tasks, source);
 
 	cal_table = e_tasks_get_calendar_table (priv->tasks);
 	etable = e_calendar_table_get_table (cal_table);
@@ -247,7 +210,7 @@ static void
 update_primary_selection (TasksComponent *component)
 {
 	TasksComponentPrivate *priv;
-	ESource *source;
+	ESource *source = NULL;
 	char *uid;
 
 	priv = component->priv;
@@ -256,29 +219,18 @@ update_primary_selection (TasksComponent *component)
 	if (uid) {
 		source = e_source_list_peek_source_by_uid (priv->source_list, uid);
 		g_free (uid);
+	}
 	
+	if (source) {
 		e_source_selector_set_primary_selection (E_SOURCE_SELECTOR (priv->source_selector), source);
 	} else {
-		ESource *source;
-		
 		/* Try to create a default if there isn't one */
-		source = find_first_source (priv->source_list);
+		source = e_source_list_peek_source_any (priv->source_list);
 		if (source)
 			e_source_selector_set_primary_selection (E_SOURCE_SELECTOR (priv->source_selector), source);
 	}
 }
 
-/* FIXME This is duplicated from comp-editor-factory.c, should it go in comp-util? */
-static ECalComponent *
-get_default_task (ECal *ecal)
-{
-	ECalComponent *comp;
-	
-	comp = cal_comp_task_new_with_defaults (ecal);
-
-	return comp;
-}
-
 /* Callbacks.  */
 static void
 add_popup_menu_item (GtkMenu *menu, const char *label, const char *pixmap,
@@ -350,14 +302,26 @@ delete_task_list_cb (GtkWidget *widget, TasksComponent *comp)
 		_("Task List '%s' will be removed. Are you sure you want to continue?"),
 		e_source_peek_name (selected_source));
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES) {
-		if (e_source_selector_source_is_selected (E_SOURCE_SELECTOR (priv->source_selector),
-							  selected_source))
-			e_source_selector_unselect_source (E_SOURCE_SELECTOR (priv->source_selector),
-							   selected_source);
+		ECal *cal;
+		char *uri;
 
-		e_source_group_remove_source (e_source_peek_group (selected_source), selected_source);
+		/* first, ask the backend to remove the task list */
+		uri = e_source_get_uri (selected_source);
+		cal = e_cal_model_get_client_for_uri (
+			e_calendar_table_get_model (E_CALENDAR_TABLE (e_tasks_get_calendar_table (priv->tasks))),
+			uri);
+		g_free (uri);
+		if (cal) {
+			if (e_cal_remove_calendar (cal, NULL)) {
+				if (e_source_selector_source_is_selected (E_SOURCE_SELECTOR (priv->source_selector),
+									  selected_source))
+					e_source_selector_unselect_source (E_SOURCE_SELECTOR (priv->source_selector),
+									   selected_source);
 
-		/* FIXME: remove the tasks.ics file and the directory */
+				e_source_group_remove_source (e_source_peek_group (selected_source), selected_source);
+				e_source_list_sync (priv->source_list, NULL);
+			}
+		}
 	}
 
 	gtk_widget_destroy (dialog);
@@ -411,6 +375,13 @@ source_selection_changed_cb (ESourceSelector *selector, TasksComponent *componen
 static void
 primary_source_selection_changed_cb (ESourceSelector *selector, TasksComponent *component)
 {
+	TasksComponentPrivate *priv = component->priv;
+
+	if (priv->create_ecal) {
+		g_object_unref (priv->create_ecal);
+		priv->create_ecal = NULL;
+	}
+
 	update_uri_for_primary_selection (component);
 }
 
@@ -459,6 +430,11 @@ impl_dispose (GObject *object)
 	g_list_free (priv->notifications);
 	priv->notifications = NULL;
 
+	if (priv->activity_handler != NULL) {
+		g_object_unref (priv->activity_handler);
+		priv->activity_handler = NULL;
+	}
+
 	(* G_OBJECT_CLASS (parent_class)->dispose) (object);
 }
 
@@ -467,6 +443,7 @@ impl_finalize (GObject *object)
 {
 	TasksComponentPrivate *priv = TASKS_COMPONENT (object)->priv;
 
+	g_free (priv->base_directory);
 	g_free (priv->config_directory);
 	g_free (priv);
 
@@ -482,49 +459,9 @@ impl_upgradeFromVersion (PortableServer_Servant servant,
 			 CORBA_short revision,
 			 CORBA_Environment *ev)
 {
-	TasksComponentPrivate *priv;
-	GSList *groups;
-	ESourceGroup *group;
-	ESource *source;
-	char *base_uri, *new_dir;
 	TasksComponent *component = TASKS_COMPONENT (bonobo_object_from_servant (servant));
 
-	priv = component->priv;
-
-	base_uri = g_build_filename (g_get_home_dir (),
-				     ".evolution/tasks/local/OnThisComputer/",
-				     NULL);
-
-	/* create default tasks folders if there are no groups */
-	groups = e_source_list_peek_groups (priv->source_list);
-	if (!groups) {
-		/* create the source group */
-		group = e_source_group_new (_("On This Computer"), base_uri);
-		e_source_list_add_group (priv->source_list, group, -1);
- 		e_source_list_sync (priv->source_list, NULL);
-	}
-
-	if (major == 1 && minor <= 4) {
-		group = e_source_list_peek_group_by_name (priv->source_list, _("On This Computer"));
-
-		/* migrate tasks from older setup */
-		if (!migrate_old_tasks (group)) {
-			/* create default tasks folders */
-			new_dir = g_build_filename (base_uri, "Personal/", NULL);
-			if (!e_mkdir_hier (new_dir, 0700)) {
-				source = e_source_new (_("Personal"), "Personal");
-				e_source_group_add_source (group, source, -1);
-			}
-
-			g_free (new_dir);
-		}
-
- 		e_source_list_sync (priv->source_list, NULL);
-	}
-
-	g_free (base_uri);
-
-	return CORBA_TRUE;
+	return migrate_tasks (component, major, minor, revision);
 }
 
 static void
@@ -536,8 +473,8 @@ impl_createControls (PortableServer_Servant servant,
 {
 	TasksComponent *component = TASKS_COMPONENT (bonobo_object_from_servant (servant));
 	TasksComponentPrivate *priv;
-	GtkWidget *selector_scrolled_window;
-	BonoboControl *sidebar_control;
+	GtkWidget *selector_scrolled_window, *statusbar_widget;
+	BonoboControl *sidebar_control, *statusbar_control;
 	guint not;
 	
 	priv = component->priv;
@@ -595,15 +532,12 @@ impl_createControls (PortableServer_Servant servant,
 	*corba_sidebar_control = CORBA_Object_duplicate (BONOBO_OBJREF (sidebar_control), ev);
 	*corba_view_control = CORBA_Object_duplicate (BONOBO_OBJREF (priv->view_control), ev);
 
-	/* The tasks component doesn't use the status bar so just return an empty label.  */
-	{
-		GtkWidget *label = gtk_label_new ("");
-		BonoboControl *control;
-
-		gtk_widget_show (label);
-		control = bonobo_control_new (label);
-		*corba_statusbar_control = CORBA_Object_duplicate (BONOBO_OBJREF (control), ev);
-	}
+	/* Create the task bar */
+	statusbar_widget = e_task_bar_new ();
+	e_activity_handler_attach_task_bar (priv->activity_handler, E_TASK_BAR (statusbar_widget));
+	gtk_widget_show (statusbar_widget);
+	statusbar_control = bonobo_control_new (statusbar_widget);
+	*corba_statusbar_control = CORBA_Object_duplicate (BONOBO_OBJREF (statusbar_control), ev);
 }
 
 static GNOME_Evolution_CreatableItemTypeList *
@@ -684,7 +618,7 @@ setup_create_ecal (TasksComponent *component)
 
 	if (!priv->create_ecal) {
 		/* Try to create a default if there isn't one */
-		source = find_first_source (priv->source_list);
+		source = e_source_list_peek_source_any (priv->source_list);
 		if (source)
 			priv->create_ecal = auth_new_cal_from_source (source, E_CAL_SOURCE_TYPE_TODO);
 	}
@@ -715,8 +649,8 @@ setup_create_ecal (TasksComponent *component)
 	}		
 
 	/* Handle the fact it may change on us */
-	not = calendar_config_add_notification_primary_calendar (config_create_ecal_changed_cb, 
-								 component);
+	not = calendar_config_add_notification_primary_tasks (config_create_ecal_changed_cb, 
+							      component);
 	priv->notifications = g_list_prepend (priv->notifications, GUINT_TO_POINTER (not));
 
 	/* Save the primary source for use elsewhere */
@@ -725,6 +659,33 @@ setup_create_ecal (TasksComponent *component)
 	return TRUE;
 }
 
+static void
+create_new_todo (TasksComponent *task_component, CORBA_Environment *ev)
+{
+	TasksComponentPrivate *priv;
+	ECalComponent *comp;
+	TaskEditor *editor;
+	gboolean read_only;
+	
+	priv = task_component->priv;
+	
+	if (!setup_create_ecal (task_component)) {
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_Failed);
+		return;
+	}
+	
+	if (!e_cal_is_read_only (priv->create_ecal, &read_only, NULL) || read_only);
+		return;
+
+	editor = task_editor_new (priv->create_ecal);
+	comp = cal_comp_task_new_with_defaults (priv->create_ecal);
+
+	comp_editor_edit_comp (COMP_EDITOR (editor), comp);
+	comp_editor_focus (COMP_EDITOR (editor));
+
+	e_comp_editor_registry_add (comp_editor_registry, COMP_EDITOR (editor), TRUE);
+}
+
 static void
 impl_requestCreateItem (PortableServer_Servant servant,
 			const CORBA_char *item_type_name,
@@ -732,28 +693,15 @@ impl_requestCreateItem (PortableServer_Servant servant,
 {
 	TasksComponent *tasks_component = TASKS_COMPONENT (bonobo_object_from_servant (servant));
 	TasksComponentPrivate *priv;
-	ECalComponent *comp;
-	TaskEditor *editor;
 	
-	priv = tasks_component->priv;
-
-	if (!setup_create_ecal (tasks_component))
-		return;	
-		
-	editor = task_editor_new (priv->create_ecal);
+	priv = tasks_component->priv;	
 	
 	if (strcmp (item_type_name, CREATE_TASK_ID) == 0) {
-		comp = get_default_task (priv->create_ecal);
-
-		comp_editor_edit_comp (COMP_EDITOR (editor), comp);
-		comp_editor_focus (COMP_EDITOR (editor));
-
-		e_comp_editor_registry_add (comp_editor_registry, COMP_EDITOR (editor), TRUE);
+		create_new_todo (tasks_component, ev);
 	} else if (strcmp (item_type_name, CREATE_TASK_LIST_ID) == 0) {
 		calendar_setup_new_task_list (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (priv->tasks))));
 	} else {
 		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_UnknownType);
-		return;
 	}
 }
 
@@ -782,6 +730,8 @@ tasks_component_init (TasksComponent *component, TasksComponentClass *klass)
 	TasksComponentPrivate *priv;
 
 	priv = g_new0 (TasksComponentPrivate, 1);
+
+	priv->base_directory = g_build_filename (g_get_home_dir (), ".evolution", NULL);
 	priv->config_directory = g_build_filename (g_get_home_dir (),
 						   ".evolution", "tasks", "config",
 						   NULL);
@@ -791,6 +741,8 @@ tasks_component_init (TasksComponent *component, TasksComponentClass *klass)
 	priv->source_list = e_source_list_new_for_gconf (priv->gconf_client,
 							 "/apps/evolution/tasks/sources");
 
+	priv->activity_handler = e_activity_handler_new ();
+
 	component->priv = priv;
 }
 
@@ -815,10 +767,34 @@ tasks_component_peek (void)
 	return component;
 }
 
+const char *
+tasks_component_peek_base_directory (TasksComponent *component)
+{
+	return component->priv->base_directory;
+}
+
 const char *
 tasks_component_peek_config_directory (TasksComponent *component)
 {
-	return (const char *) component->priv->config_directory;
+	return component->priv->config_directory;
+}
+
+ESourceList *
+tasks_component_peek_source_list (TasksComponent *component)
+{
+	return component->priv->source_list;	
+}
+
+ESourceSelector *
+tasks_component_peek_source_selector (TasksComponent *component)
+{
+	return component->priv->source_selector;
+}
+
+EActivityHandler *
+tasks_component_peek_activity_handler (TasksComponent *component)
+{
+	return component->priv->activity_handler;
 }
 
 BONOBO_TYPE_FUNC_FULL (TasksComponent, GNOME_Evolution_Component, PARENT_TYPE, tasks_component)
