@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* tasks-component.h
  *
- * Copyright (C) 2003  Rodrigo Moya
+ * Copyright (C) 2003  Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -25,6 +25,8 @@
 
 #include <bonobo/bonobo-object.h>
 #include <libedataserver/e-source-list.h>
+#include <shell/e-activity-handler.h>
+#include <widgets/misc/e-source-selector.h>
 #include "Evolution.h"
 
 
@@ -52,9 +54,13 @@ struct _TasksComponentClass {
 };
 
 
-GType           tasks_component_get_type  (void);
-TasksComponent *tasks_component_peek  (void);
-const char     *tasks_component_peek_config_directory  (TasksComponent *component);
-ESourceList    *tasks_component_peek_source_list (TasksComponent *component);
+GType             tasks_component_get_type  (void);
+TasksComponent   *tasks_component_peek  (void);
+
+const char       *tasks_component_peek_base_directory (TasksComponent *component);
+const char       *tasks_component_peek_config_directory (TasksComponent *component);
+ESourceList      *tasks_component_peek_source_list (TasksComponent *component);
+ESourceSelector  *tasks_component_peek_source_selector (TasksComponent *component);
+EActivityHandler *tasks_component_peek_activity_handler (TasksComponent *component);
 
 #endif /* _TASKS_COMPONENT_H_ */
