@@ -6,7 +6,7 @@
  *	    Michael Zucchi <notzed@ximian.com>
  *          Jeffrey Stedfast <fejj@ximian.com>
  *
- * Copyright 1999, 2000 Ximian, Inc. (www.ximian.com)
+ * Copyright 1999-2003 Ximian, Inc. (www.ximian.com)
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of version 2 of the GNU General Public 
@@ -30,16 +30,16 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
 
 #include <gal/util/e-iconv.h>
 
-#include <errno.h>
+#include <e-util/e-time-utils.h>
 
 #include "camel-mime-message.h"
 #include "camel-multipart.h"
 #include "camel-stream-mem.h"
-#include "string-utils.h"
-#include "hash-table-utils.h"
+#include "camel-string-utils.h"
 #include "camel-url.h"
 
 #include "camel-stream-filter.h"
@@ -56,29 +56,33 @@ typedef enum {
 	HEADER_REPLY_TO,
 	HEADER_SUBJECT,
 	HEADER_TO,
+	HEADER_RESENT_TO,
 	HEADER_CC,
+	HEADER_RESENT_CC,
 	HEADER_BCC,
+	HEADER_RESENT_BCC,
 	HEADER_DATE,
 	HEADER_MESSAGE_ID
 } CamelHeaderType;
 
 static char *header_names[] = {
 	/* dont include HEADER_UNKNOWN string */
-	"From", "Reply-To", "Subject", "To", "Cc", "Bcc", "Date", "Message-Id", NULL
+	"From", "Reply-To", "Subject", "To", "Resent-To", "Cc", "Resent-Cc",
+	"Bcc", "Resent-Bcc", "Date", "Message-Id", NULL
 };
 
 static GHashTable *header_name_table;
 
-static CamelMimePartClass *parent_class=NULL;
+static CamelMimePartClass *parent_class = NULL;
 
 static char *recipient_names[] = {
-	"To", "Cc", "Bcc", NULL
+	"To", "Cc", "Bcc", "Resent-To", "Resent-Cc", "Resent-Bcc", NULL
 };
 
-static int write_to_stream (CamelDataWrapper *data_wrapper, CamelStream *stream);
-static void add_header (CamelMedium *medium, const char *header_name, const void *header_value);
-static void set_header (CamelMedium *medium, const char *header_name, const void *header_value);
-static void remove_header (CamelMedium *medium, const char *header_name);
+static ssize_t write_to_stream (CamelDataWrapper *data_wrapper, CamelStream *stream);
+static void add_header (CamelMedium *medium, const char *name, const void *value);
+static void set_header (CamelMedium *medium, const char *name, const void *value);
+static void remove_header (CamelMedium *medium, const char *name);
 static int construct_from_parser (CamelMimePart *, CamelMimeParser *);
 static void unref_recipient (gpointer key, gpointer value, gpointer user_data);
 
@@ -95,14 +99,15 @@ camel_mime_message_class_init (CamelMimeMessageClass *camel_mime_message_class)
 	CamelMediumClass *camel_medium_class = CAMEL_MEDIUM_CLASS (camel_mime_message_class);
 	int i;
 	
-	parent_class = CAMEL_MIME_PART_CLASS(camel_type_get_global_classfuncs (camel_mime_part_get_type ()));
-
-	header_name_table = g_hash_table_new (g_strcase_hash, g_strcase_equal);
-	for (i=0;header_names[i];i++)
-		g_hash_table_insert (header_name_table, header_names[i], (gpointer)i+1);
+	parent_class = CAMEL_MIME_PART_CLASS (camel_type_get_global_classfuncs (camel_mime_part_get_type ()));
+	
+	header_name_table = g_hash_table_new (camel_strcase_hash, camel_strcase_equal);
+	for (i = 0;header_names[i]; i++)
+		g_hash_table_insert (header_name_table, header_names[i], GINT_TO_POINTER(i+1));
 
 	/* virtual method overload */
 	camel_data_wrapper_class->write_to_stream = write_to_stream;
+	camel_data_wrapper_class->decode_to_stream = write_to_stream;
 
 	camel_medium_class->add_header = add_header;
 	camel_medium_class->set_header = set_header;
@@ -118,9 +123,7 @@ camel_mime_message_init (gpointer object, gpointer klass)
 	CamelMimeMessage *mime_message = (CamelMimeMessage *)object;
 	int i;
 	
-	camel_data_wrapper_set_mime_type (CAMEL_DATA_WRAPPER (object), "message/rfc822");
-
-	mime_message->recipients =  g_hash_table_new(g_strcase_hash, g_strcase_equal);
+	mime_message->recipients =  g_hash_table_new (camel_strcase_hash, camel_strcase_equal);
 	for (i=0;recipient_names[i];i++) {
 		g_hash_table_insert(mime_message->recipients, recipient_names[i], camel_internet_address_new());
 	}
@@ -176,7 +179,7 @@ camel_mime_message_get_type (void)
 static void
 unref_recipient (gpointer key, gpointer value, gpointer user_data)
 {
-	camel_object_unref (CAMEL_OBJECT (value));
+	camel_object_unref (value);
 }
 
 CamelMimeMessage *
@@ -198,27 +201,17 @@ camel_mime_message_set_date (CamelMimeMessage *message,  time_t date, int offset
 	g_assert(message);
 	
 	if (date == CAMEL_MESSAGE_DATE_CURRENT) {
-		struct tm *local;
+		struct tm local;
 		int tz;
 		
 		date = time(0);
-		local = localtime(&date);
-#if defined(HAVE_TIMEZONE)
-		tz = timezone;
-#elif defined(HAVE_TM_GMTOFF)
-		tz = -local->tm_gmtoff;
-#endif
-		offset = -(((tz/60/60) * 100) + (tz/60 % 60));
-#ifdef HAVE_TIMEZONE
-		/* tm.tm_gmtoff is already adjusted for DST */
-		if (local->tm_isdst>0)
-			offset += 100;
-#endif
+		e_localtime_with_offset(date, &local, &tz);
+		offset = (((tz/60/60) * 100) + (tz/60 % 60));
 	}
 	message->date = date;
 	message->date_offset = offset;
 	
-	datestr = header_format_date (date, offset);
+	datestr = camel_header_format_date (date, offset);
 	CAMEL_MEDIUM_CLASS (parent_class)->set_header ((CamelMedium *)message, "Date", datestr);
 	g_free (datestr);
 }
@@ -242,7 +235,7 @@ camel_mime_message_get_date_received (CamelMimeMessage *msg, int *offset)
 		if (received)
 			received = strrchr (received, ';');
 		if (received)
-			msg->date_received = header_decode_date (received + 1, &msg->date_received_offset);
+			msg->date_received = camel_header_decode_date (received + 1, &msg->date_received_offset);
 	}
 	
 	if (offset)
@@ -265,7 +258,7 @@ camel_mime_message_set_message_id (CamelMimeMessage *mime_message, const char *m
 	if (message_id) {
 		id = g_strstrip (g_strdup (message_id));
 	} else {
-		id = header_msgid_generate ();
+		id = camel_header_msgid_generate ();
 	}
 	
 	mime_message->message_id = id;
@@ -328,7 +321,7 @@ camel_mime_message_set_subject (CamelMimeMessage *mime_message, const char *subj
 	
 	g_free (mime_message->subject);
 	mime_message->subject = g_strstrip (g_strdup (subject));
-	text = header_encode_string((unsigned char *)mime_message->subject);
+	text = camel_header_encode_string((unsigned char *)mime_message->subject);
 	CAMEL_MEDIUM_CLASS(parent_class)->set_header(CAMEL_MEDIUM (mime_message), "Subject", text);
 	g_free (text);
 }
@@ -455,7 +448,7 @@ static int
 construct_from_parser (CamelMimePart *dw, CamelMimeParser *mp)
 {
 	char *buf;
-	int len;
+	size_t len;
 	int state;
 	int ret;
 	int err;
@@ -473,9 +466,9 @@ construct_from_parser (CamelMimePart *dw, CamelMimeParser *mp)
 	/* ... then clean up the follow-on state */
 	state = camel_mime_parser_step (mp, &buf, &len);
 	switch (state) {
-	case HSCAN_EOF: case HSCAN_FROM_END: /* these doesn't belong to us */
+	case CAMEL_MIME_PARSER_STATE_EOF: case CAMEL_MIME_PARSER_STATE_FROM_END: /* these doesn't belong to us */
 		camel_mime_parser_unstep (mp);
-	case HSCAN_MESSAGE_END:
+	case CAMEL_MIME_PARSER_STATE_MESSAGE_END:
 		break;
 	default:
 		g_error ("Bad parser state: Expecing MESSAGE_END or EOF or EOM, got: %d", camel_mime_parser_state (mp));
@@ -484,9 +477,6 @@ construct_from_parser (CamelMimePart *dw, CamelMimeParser *mp)
 	}
 
 	d(printf("mime_message::construct_from_parser() leaving\n"));
-#ifndef NO_WARNINGS
-#warning "return a real error code"
-#endif
 	err = camel_mime_parser_errno(mp);
 	if (err != 0) {
 		errno = err;
@@ -496,7 +486,7 @@ construct_from_parser (CamelMimePart *dw, CamelMimeParser *mp)
 	return ret;
 }
 
-static int
+static ssize_t
 write_to_stream (CamelDataWrapper *data_wrapper, CamelStream *stream)
 {
 	CamelMimeMessage *mm = CAMEL_MIME_MESSAGE (data_wrapper);
@@ -526,47 +516,51 @@ write_to_stream (CamelDataWrapper *data_wrapper, CamelStream *stream)
 
 /* FIXME: check format of fields. */
 static gboolean
-process_header (CamelMedium *medium, const char *header_name, const char *header_value)
+process_header (CamelMedium *medium, const char *name, const char *value)
 {
 	CamelHeaderType header_type;
 	CamelMimeMessage *message = CAMEL_MIME_MESSAGE (medium);
 	CamelInternetAddress *addr;
 	const char *charset;
 	
-	header_type = (CamelHeaderType)g_hash_table_lookup (header_name_table, header_name);
+	header_type = (CamelHeaderType) g_hash_table_lookup (header_name_table, name);
 	switch (header_type) {
 	case HEADER_FROM:
 		if (message->from)
-			camel_object_unref (CAMEL_OBJECT (message->from));
+			camel_object_unref (message->from);
 		message->from = camel_internet_address_new ();
-		camel_address_decode (CAMEL_ADDRESS (message->from), header_value);
+		camel_address_decode (CAMEL_ADDRESS (message->from), value);
 		break;
 	case HEADER_REPLY_TO:
 		if (message->reply_to)
-			camel_object_unref (CAMEL_OBJECT (message->reply_to));
+			camel_object_unref (message->reply_to);
 		message->reply_to = camel_internet_address_new ();
-		camel_address_decode (CAMEL_ADDRESS (message->reply_to), header_value);
+		camel_address_decode (CAMEL_ADDRESS (message->reply_to), value);
 		break;
 	case HEADER_SUBJECT:
 		g_free (message->subject);
-		if (((CamelMimePart *)message)->content_type)
-			charset = e_iconv_charset_name(header_content_type_param(((CamelMimePart *)message)->content_type, "charset"));
-		else
+		if (((CamelDataWrapper *) message)->mime_type) {
+			charset = camel_content_type_param (((CamelDataWrapper *) message)->mime_type, "charset");
+			charset = e_iconv_charset_name (charset);
+		} else
 			charset = NULL;
-		message->subject = g_strstrip (header_decode_string (header_value, charset));
+		message->subject = g_strstrip (camel_header_decode_string (value, charset));
 		break;
 	case HEADER_TO:
 	case HEADER_CC:
 	case HEADER_BCC:
-		addr = g_hash_table_lookup (message->recipients, header_name);
-		if (header_value)
-			camel_address_decode (CAMEL_ADDRESS (addr), header_value);
+	case HEADER_RESENT_TO:
+	case HEADER_RESENT_CC:
+	case HEADER_RESENT_BCC:
+		addr = g_hash_table_lookup (message->recipients, name);
+		if (value)
+			camel_address_decode (CAMEL_ADDRESS (addr), value);
 		else
 			camel_address_remove (CAMEL_ADDRESS (addr), -1);
 		break;
 	case HEADER_DATE:
-		if (header_value) {
-			message->date = header_decode_date (header_value, &message->date_offset);
+		if (value) {
+			message->date = camel_header_decode_date (value, &message->date_offset);
 		} else {
 			message->date = CAMEL_MESSAGE_DATE_CURRENT;
 			message->date_offset = 0;
@@ -574,8 +568,8 @@ process_header (CamelMedium *medium, const char *header_name, const char *header
 		break;
 	case HEADER_MESSAGE_ID:
 		g_free (message->message_id);
-		if (header_value)
-			message->message_id = header_msgid_decode (header_value);
+		if (value)
+			message->message_id = camel_header_msgid_decode (value);
 		else
 			message->message_id = NULL;
 		break;
@@ -587,27 +581,27 @@ process_header (CamelMedium *medium, const char *header_name, const char *header
 }
 
 static void
-set_header (CamelMedium *medium, const char *header_name, const void *header_value)
+set_header (CamelMedium *medium, const char *name, const void *value)
 {
-	process_header (medium, header_name, header_value);
-	parent_class->parent_class.set_header (medium, header_name, header_value);
+	process_header (medium, name, value);
+	parent_class->parent_class.set_header (medium, name, value);
 }
 
 static void
-add_header (CamelMedium *medium, const char *header_name, const void *header_value)
+add_header (CamelMedium *medium, const char *name, const void *value)
 {
 	/* if we process it, then it must be forced unique as well ... */
-	if (process_header (medium, header_name, header_value))
-		parent_class->parent_class.set_header (medium, header_name, header_value);
+	if (process_header (medium, name, value))
+		parent_class->parent_class.set_header (medium, name, value);
 	else
-		parent_class->parent_class.add_header (medium, header_name, header_value);
+		parent_class->parent_class.add_header (medium, name, value);
 }
 
 static void
-remove_header (CamelMedium *medium, const char *header_name)
+remove_header (CamelMedium *medium, const char *name)
 {
-	process_header (medium, header_name, NULL);
-	parent_class->parent_class.remove_header (medium, header_name);
+	process_header (medium, name, NULL);
+	parent_class->parent_class.remove_header (medium, name);
 }
 
 typedef gboolean (*CamelPartFunc)(CamelMimeMessage *, CamelMimePart *, void *data);
@@ -653,10 +647,14 @@ camel_mime_message_foreach_part (CamelMimeMessage *msg, CamelPartFunc callback,
 static gboolean
 check_8bit (CamelMimeMessage *msg, CamelMimePart *part, void *data)
 {
+	CamelTransferEncoding encoding;
 	int *has8bit = data;
 	
 	/* check this part, and stop as soon as we are done */
-	*has8bit = camel_mime_part_get_encoding (part) == CAMEL_MIME_PART_ENCODING_8BIT;
+	encoding = camel_mime_part_get_encoding (part);
+	
+	*has8bit = encoding == CAMEL_TRANSFER_ENCODING_8BIT || encoding == CAMEL_TRANSFER_ENCODING_BINARY;
+	
 	return !(*has8bit);
 }
 
@@ -671,20 +669,20 @@ camel_mime_message_has_8bit_parts (CamelMimeMessage *msg)
 }
 
 /* finds the best charset and transfer encoding for a given part */
-static CamelMimePartEncodingType
+static CamelTransferEncoding
 find_best_encoding (CamelMimePart *part, CamelBestencRequired required, CamelBestencEncoding enctype, char **charsetp)
 {
+	CamelMimeFilterCharset *charenc = NULL;
+	CamelTransferEncoding encoding;
+	CamelMimeFilterBestenc *bestenc;
+	unsigned int flags, callerflags;
+	CamelDataWrapper *content;
+	CamelStreamFilter *filter;
 	const char *charsetin = NULL;
 	char *charset = NULL;
 	CamelStream *null;
-	CamelStreamFilter *filter;
-	CamelMimeFilterCharset *charenc = NULL;
-	CamelMimeFilterBestenc *bestenc;
 	int idb, idc = -1;
 	gboolean istext;
-	unsigned int flags, callerflags;
-	CamelMimePartEncodingType encoding;
-	CamelDataWrapper *content;
 	
 	/* we use all these weird stream things so we can do it with streams, and
 	   not have to read the whole lot into memory - although i have a feeling
@@ -697,12 +695,13 @@ find_best_encoding (CamelMimePart *part, CamelBestencRequired required, CamelBes
 		/* charset might not be right here, but it'll get the right stuff
 		   if it is ever set */
 		*charsetp = NULL;
-		return CAMEL_MIME_PART_ENCODING_DEFAULT;
+		return CAMEL_TRANSFER_ENCODING_DEFAULT;
 	}
 	
-	istext = header_content_type_is (part->content_type, "text", "*");
+	istext = camel_content_type_is (((CamelDataWrapper *) part)->mime_type, "text", "*");
 	if (istext) {
 		flags = CAMEL_BESTENC_GET_CHARSET | CAMEL_BESTENC_GET_ENCODING;
+		enctype |= CAMEL_BESTENC_TEXT;
 	} else {
 		flags = CAMEL_BESTENC_GET_ENCODING;
 	}
@@ -717,12 +716,10 @@ find_best_encoding (CamelMimePart *part, CamelBestencRequired required, CamelBes
 	null = (CamelStream *)camel_stream_null_new ();
 	filter = camel_stream_filter_new_with_stream (null);
 	
-	/* if we're not looking for the best charset, then use the one we have */
-	if (istext && (required & CAMEL_BESTENC_GET_CHARSET) == 0
-	    && (charsetin = header_content_type_param (part->content_type, "charset"))) {
-		/* if libunicode doesn't support it, we dont really have utf8 anyway, so
-		   we dont need a converter */
-		charenc = camel_mime_filter_charset_new_convert ("UTF-8", charsetin);
+	/* if we're looking for the best charset, then we need to convert to UTF-8 */
+	if (istext && (required & CAMEL_BESTENC_GET_CHARSET) != 0
+	    && (charsetin = camel_content_type_param (content->mime_type, "charset"))) {
+		charenc = camel_mime_filter_charset_new_convert (charsetin, "UTF-8");
 		if (charenc != NULL)
 			idc = camel_stream_filter_add (filter, (CamelMimeFilter *)charenc);
 		charsetin = NULL;
@@ -731,35 +728,33 @@ find_best_encoding (CamelMimePart *part, CamelBestencRequired required, CamelBes
 	bestenc = camel_mime_filter_bestenc_new (flags);
 	idb = camel_stream_filter_add (filter, (CamelMimeFilter *)bestenc);
 	d(printf("writing to checking stream\n"));
-	camel_data_wrapper_write_to_stream (content, (CamelStream *)filter);
+	camel_data_wrapper_decode_to_stream (content, (CamelStream *)filter);
 	camel_stream_filter_remove (filter, idb);
 	if (idc != -1) {
 		camel_stream_filter_remove (filter, idc);
-		camel_object_unref ((CamelObject *)charenc);
+		camel_object_unref (charenc);
 		charenc = NULL;
 	}
 	
-	if (istext)
+	if (istext && (required & CAMEL_BESTENC_GET_CHARSET) != 0) {
 		charsetin = camel_mime_filter_bestenc_get_best_charset (bestenc);
-	
-	d(printf("charsetin = %s\n", charsetin ? charsetin : "(null)"));
-	
-	/* if we have US-ASCII, or we're not doing text, we dont need to bother with the rest */
-	if (charsetin != NULL && (required & CAMEL_BESTENC_GET_CHARSET) != 0) {
+		d(printf("best charset = %s\n", charsetin ? charsetin : "(null)"));
 		charset = g_strdup (charsetin);
 		
+		charsetin = camel_content_type_param (content->mime_type, "charset");
+	} else {
+		charset = NULL;
+	}
+	
+	/* if we have US-ASCII, or we're not doing text, we dont need to bother with the rest */
+	if (istext && charsetin && charset && (required & CAMEL_BESTENC_GET_CHARSET) != 0) {
 		d(printf("have charset, trying conversion/etc\n"));
 		
-		/* now the 'bestenc' can has told us what the best encoding is, we can use that to create
+		/* now that 'bestenc' has told us what the best encoding is, we can use that to create
 		   a charset conversion filter as well, and then re-add the bestenc to filter the
 		   result to find the best encoding to use as well */
 		
-		charenc = camel_mime_filter_charset_new_convert ("UTF-8", charset);
-		
-		/* eek, libunicode doesn't undertand this charset anyway, then the 'utf8' we
-		   thought we had is really the native format, in which case, we just treat
-		   it as binary data (and take the result we have so far) */
-		
+		charenc = camel_mime_filter_charset_new_convert (charsetin, charset);
 		if (charenc != NULL) {
 			/* otherwise, try another pass, converting to the real charset */
 			
@@ -773,15 +768,15 @@ find_best_encoding (CamelMimePart *part, CamelBestencRequired required, CamelBes
 			/* and write it to the new stream */
 			camel_data_wrapper_write_to_stream (content, (CamelStream *)filter);
 			
-			camel_object_unref ((CamelObject *)charenc);
+			camel_object_unref (charenc);
 		}
 	}
 	
 	encoding = camel_mime_filter_bestenc_get_best_encoding (bestenc, enctype);
 	
-	camel_object_unref ((CamelObject *)filter);
-	camel_object_unref ((CamelObject *)bestenc);
-	camel_object_unref ((CamelObject *)null);
+	camel_object_unref (filter);
+	camel_object_unref (bestenc);
+	camel_object_unref (null);
 	
 	d(printf("done, best encoding = %d\n", encoding));
 	
@@ -802,7 +797,7 @@ static gboolean
 best_encoding (CamelMimeMessage *msg, CamelMimePart *part, void *datap)
 {
 	struct _enc_data *data = datap;
-	CamelMimePartEncodingType encoding;
+	CamelTransferEncoding encoding;
 	CamelDataWrapper *wrapper;
 	char *charset;
 	
@@ -818,13 +813,13 @@ best_encoding (CamelMimeMessage *msg, CamelMimePart *part, void *datap)
 		camel_mime_part_set_encoding (part, encoding);
 		
 		if ((data->required & CAMEL_BESTENC_GET_CHARSET) != 0) {
-			if (header_content_type_is (part->content_type, "text", "*")) {
+			if (camel_content_type_is (((CamelDataWrapper *) part)->mime_type, "text", "*")) {
 				char *newct;
 				
 				/* FIXME: ick, the part content_type interface needs fixing bigtime */
-				header_content_type_set_param (part->content_type, "charset",
+				camel_content_type_set_param (((CamelDataWrapper *) part)->mime_type, "charset",
 							       charset ? charset : "us-ascii");
-				newct = header_content_type_format (part->content_type);
+				newct = camel_content_type_format (((CamelDataWrapper *) part)->mime_type);
 				if (newct) {
 					d(printf("Setting content-type to %s\n", newct));
 					
@@ -833,6 +828,8 @@ best_encoding (CamelMimeMessage *msg, CamelMimePart *part, void *datap)
 				}
 			}
 		}
+		
+		g_free (charset);
 	}
 	
 	return TRUE;
@@ -876,7 +873,7 @@ check_content_id (CamelMimeMessage *message, CamelMimePart *part, void *data)
 	found = content_id && !strcmp (content_id, check->content_id) ? TRUE : FALSE;
 	if (found) {
 		check->part = part;
-		camel_object_ref (CAMEL_OBJECT (part));
+		camel_object_ref (part);
 	}
 	
 	return !found;
@@ -899,3 +896,68 @@ camel_mime_message_get_part_by_content_id (CamelMimeMessage *message, const char
 	
 	return check.part;
 }
+
+static char *tz_months[] = {
+	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+};
+
+static char *tz_days[] = {
+	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
+};
+
+char *
+camel_mime_message_build_mbox_from (CamelMimeMessage *message)
+{
+	struct _camel_header_raw *header = ((CamelMimePart *)message)->headers;
+	GString *out = g_string_new("From ");
+	char *ret;
+	const char *tmp;
+	time_t thetime;
+	int offset;
+	struct tm tm;
+	
+	tmp = camel_header_raw_find (&header, "Sender", NULL);
+	if (tmp == NULL)
+		tmp = camel_header_raw_find (&header, "From", NULL);
+	if (tmp != NULL) {
+		struct _camel_header_address *addr = camel_header_address_decode (tmp, NULL);
+		
+		tmp = NULL;
+		if (addr) {
+			if (addr->type == CAMEL_HEADER_ADDRESS_NAME) {
+				g_string_append (out, addr->v.addr);
+				tmp = "";
+			}
+			camel_header_address_unref (addr);
+		}
+	}
+	
+	if (tmp == NULL)
+		g_string_append (out, "unknown@nodomain.now.au");
+	
+	/* try use the received header to get the date */
+	tmp = camel_header_raw_find (&header, "Received", NULL);
+	if (tmp) {
+		tmp = strrchr(tmp, ';');
+		if (tmp)
+			tmp++;
+	}
+	
+	/* if there isn't one, try the Date field */
+	if (tmp == NULL)
+		tmp = camel_header_raw_find (&header, "Date", NULL);
+	
+	thetime = camel_header_decode_date (tmp, &offset);
+	thetime += ((offset / 100) * (60 * 60)) + (offset % 100) * 60;
+	gmtime_r (&thetime, &tm);
+	g_string_append_printf (out, " %s %s %2d %02d:%02d:%02d %4d\n",
+				tz_days[tm.tm_wday], tz_months[tm.tm_mon],
+				tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec,
+				tm.tm_year + 1900);
+	
+	ret = out->str;
+	g_string_free (out, FALSE);
+	
+	return ret;
+}
