@@ -307,7 +307,10 @@ camel_imap_folder_selected (CamelFolder *folder, CamelImapResponse *response,
 			}
 		}
 	}
-	
+
+	if (camel_strstrcase (response->status, "OK [READ-ONLY]"))
+		imap_folder->read_only = TRUE;
+
 	if (camel_disco_store_status (CAMEL_DISCO_STORE (folder->parent_store)) == CAMEL_DISCO_STORE_RESYNCING) {
 		if (validity != imap_summary->validity) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_SUMMARY_INVALID,
@@ -752,6 +755,11 @@ imap_sync_online (CamelFolder *folder, CamelException *ex)
 	gboolean unset;
 	int i, j, max;
 	
+	if (((CamelImapFolder *)folder)->read_only) {
+		imap_sync_offline (folder, ex);
+		return;
+	}
+
 	camel_exception_init (&local_ex);
 	CAMEL_SERVICE_LOCK (store, connect_lock);
 	
@@ -761,9 +769,9 @@ imap_sync_online (CamelFolder *folder, CamelException *ex)
 	 */
 	max = camel_folder_summary_count (folder->summary);
 	for (i = 0; i < max; i++) {
-		info = camel_folder_summary_index (folder->summary, i);
-		if (!info)
+		if (!(info = camel_folder_summary_index (folder->summary, i)))
 			continue;
+		
 		if (!(info->flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
 			camel_folder_summary_info_free (folder->summary, info);
 			continue;
@@ -772,23 +780,23 @@ imap_sync_online (CamelFolder *folder, CamelException *ex)
 		/* Note: Cyrus is broken and will not accept an
 		   empty-set of flags so... if this is true then we
 		   want to unset the previously set flags.*/
-		unset = !(info->flags & CAMEL_IMAP_SERVER_FLAGS);
-				
+		unset = !(info->flags & folder->permanent_flags);
+		
 		/* Note: get_matching() uses UID_SET_LIMIT to limit
 		   the size of the uid-set string. We don't have to
 		   loop here to flush all the matching uids because
 		   they will be scooped up later by our parent loop (I
 		   think?). -- Jeff */
-		matches = get_matching (folder, info->flags & (CAMEL_IMAP_SERVER_FLAGS | CAMEL_MESSAGE_FOLDER_FLAGGED),
-					CAMEL_IMAP_SERVER_FLAGS | CAMEL_MESSAGE_FOLDER_FLAGGED, &set);
+		matches = get_matching (folder, info->flags & (folder->permanent_flags | CAMEL_MESSAGE_FOLDER_FLAGGED),
+					folder->permanent_flags | CAMEL_MESSAGE_FOLDER_FLAGGED, &set);
 		camel_folder_summary_info_free (folder->summary, info);
 		if (matches == NULL)
 			continue;
 		
 		/* FIXME: since we don't know the previously set flags,
 		   if unset is TRUE then just unset all the flags? */
-		flaglist = imap_create_flag_list (unset ? CAMEL_IMAP_SERVER_FLAGS : info->flags);
-
+		flaglist = imap_create_flag_list (unset ? folder->permanent_flags : info->flags & folder->permanent_flags);
+		
 		/* Note: to `unset' flags, use -FLAGS.SILENT (<flag list>) */
 		response = camel_imap_command (store, folder, &local_ex,
 					       "UID STORE %s %sFLAGS.SILENT %s",
@@ -803,7 +811,7 @@ imap_sync_online (CamelFolder *folder, CamelException *ex)
 			for (j = 0; j < matches->len; j++) {
 				info = matches->pdata[j];
 				info->flags &= ~CAMEL_MESSAGE_FOLDER_FLAGGED;
-				((CamelImapMessageInfo*)info)->server_flags =
+				((CamelImapMessageInfo *) info)->server_flags =
 					info->flags & CAMEL_IMAP_SERVER_FLAGS;
 			}
 			camel_folder_summary_touch (folder->summary);
@@ -925,11 +933,15 @@ imap_expunge_uids_online (CamelFolder *folder, GPtrArray *uids, CamelException *
 static void
 imap_expunge_uids_resyncing (CamelFolder *folder, GPtrArray *uids, CamelException *ex)
 {
+	CamelImapFolder *imap_folder = CAMEL_IMAP_FOLDER (folder);
 	CamelImapStore *store = CAMEL_IMAP_STORE (folder->parent_store);
 	GPtrArray *keep_uids, *mark_uids;
 	CamelImapResponse *response;
 	char *result;
 
+	if (imap_folder->read_only)
+		return;
+
 	if (store->capabilities & IMAP_CAPABILITY_UIDPLUS) {
 		imap_expunge_uids_online (folder, uids, ex);
 		return;
@@ -1871,7 +1883,6 @@ get_message (CamelImapFolder *imap_folder, const char *uid,
 	return msg;
 }
 
-/* FIXME: I pulled this number out of my butt. */
 #define IMAP_SMALL_BODY_SIZE 5120
 
 static CamelMimeMessage *
@@ -1924,12 +1935,6 @@ imap_get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 	    (stream = camel_imap_folder_fetch_data (imap_folder, uid, "", TRUE, NULL)))
 		return get_message_simple (imap_folder, uid, stream, ex);
 
-	/* If we're not actually connected and it's not in the cache,
-	 * that's as far as we can go.
-	 */
-	if (camel_disco_store_check_online (CAMEL_DISCO_STORE (store), ex) == FALSE)
-		return NULL;
-
 	mi = camel_folder_summary_uid (folder->summary, uid);
 	if (mi == NULL) {
 		camel_exception_setv(ex, CAMEL_EXCEPTION_FOLDER_INVALID_UID,
