@@ -50,7 +50,6 @@ static void construct (CamelService *service, CamelSession *session, CamelProvid
 static CamelFolder *get_folder(CamelStore * store, const char *folder_name, guint32 flags, CamelException * ex);
 static char *get_name(CamelService *service, gboolean brief);
 static CamelFolder *get_inbox (CamelStore *store, CamelException *ex);
-static void rename_folder(CamelStore *store, const char *old_name, const char *new_name, CamelException *ex);
 static CamelFolderInfo *get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelException *ex);
 static void delete_folder(CamelStore *store, const char *folder_name, CamelException *ex);
 static void rename_folder(CamelStore *store, const char *old, const char *new, CamelException *ex);
@@ -151,24 +150,25 @@ get_folder(CamelStore * store, const char *folder_name, guint32 flags, CamelExce
 
 	if (errno != ENOENT
 	    || (flags & CAMEL_STORE_FOLDER_CREATE) == 0) {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
-				     _("Cannot get folder: %s: %s"), path, strerror(errno));
+		camel_exception_setv (ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+				      _("Cannot get folder: %s: %s"),
+				      path, g_strerror (errno));
 		return NULL;
 	}
 
 	/* need to create the dir heirarchy */
-	sub = alloca(strlen(path)+1);
-	strcpy(sub, path);
+	sub = g_alloca (strlen (path) + 1);
+	strcpy (sub, path);
 	slash = sub;
 	do {
-		slash = strchr(slash+1, '/');
+		slash = strchr (slash + 1, '/');
 		if (slash)
 			*slash = 0;
-		if (stat(sub, &st) == -1) {
-			if (errno != ENOENT
-			    || mkdir(sub, 0700) == -1) {
-				camel_exception_setv(ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
-						     _("Cannot get folder: %s: %s"), path, strerror(errno));
+		if (stat (sub, &st) == -1) {
+			if (errno != ENOENT || mkdir (sub, 0700) == -1) {
+				camel_exception_setv (ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+						      _("Cannot get folder: %s: %s"),
+						      path, g_strerror (errno));
 				return NULL;
 			}
 		}
@@ -206,7 +206,7 @@ get_folder_info (CamelStore *store, const char *top,
 	 * there before.
 	 */
 
-	d(printf("-- LOCAL STRE -- get folder info: %s\n", top));
+	d(printf("-- LOCAL STORE -- get folder info: %s\n", top));
 
 	return NULL;
 }
@@ -222,26 +222,31 @@ create_folder(CamelStore *store, const char *parent_name, const char *folder_nam
 
 	/* This is a pretty hacky version of create folder, but should basically work */
 
-	/* FIXME: The strings here are funny because of string freeze */
-
 	if (path[0] != '/') {
 		camel_exception_setv(ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
 				     _("Store root %s is not an absolute path"), path);
 		return NULL;
 	}
 
-	name = g_strdup_printf("%s/%s/%s", path, parent_name, folder_name);
+	if (parent_name)
+		name = g_strdup_printf("%s/%s/%s", path, parent_name, folder_name);
+	else
+		name = g_strdup_printf("%s/%s", path, folder_name);
 
 	if (stat(name, &st) == 0 || errno != ENOENT) {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
-				     _("Cannot get folder: %s: %s"), name, strerror(errno));
+		camel_exception_setv (ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+				      _("Cannot get folder: %s: %s"),
+				      name, g_strerror (errno));
 		g_free(name);
 		return NULL;
 	}
 
 	g_free(name);
 
-	name = g_strdup_printf("%s/%s", parent_name, folder_name);
+	if (parent_name)
+		name = g_strdup_printf("%s/%s", parent_name, folder_name);
+	else
+		name = g_strdup_printf("%s", folder_name);
 
 	folder = ((CamelStoreClass *)((CamelObject *)store)->klass)->get_folder(store, name, CAMEL_STORE_FOLDER_CREATE, ex);
 	if (folder) {
@@ -298,9 +303,9 @@ static int xrename(const char *oldp, const char *newp, const char *prefix, const
 	}
 
 	if (ret == -1) {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
-				     _("Could not rename folder %s to %s: %s"),
-				     old, new, strerror(err));
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Could not rename folder %s to %s: %s"),
+				      old, new, g_strerror (err));
 	}
 
 	g_free(old);
@@ -313,7 +318,7 @@ static void
 rename_folder(CamelStore *store, const char *old, const char *new, CamelException *ex)
 {
 	char *path = CAMEL_LOCAL_STORE (store)->toplevel_dir;
-	CamelLocalFolder *folder;
+	CamelLocalFolder *folder = NULL;
 	char *newibex = g_strdup_printf("%s%s.ibex", path, new);
 	char *oldibex = g_strdup_printf("%s%s.ibex", path, old);
 
@@ -321,8 +326,7 @@ rename_folder(CamelStore *store, const char *old, const char *new, CamelExceptio
 
 	d(printf("local rename folder '%s' '%s'\n", old, new));
 
-	CAMEL_STORE_LOCK(store, cache_lock);
-	folder = g_hash_table_lookup(store->folders, old);
+	folder = camel_object_bag_get(store->folders, old);
 	if (folder && folder->index) {
 		if (camel_index_rename(folder->index, newibex) == -1)
 			goto ibex_failed;
@@ -338,11 +342,12 @@ rename_folder(CamelStore *store, const char *old, const char *new, CamelExceptio
 	if (xrename(old, new, path, "", FALSE, ex))
 		goto base_failed;
 
-	CAMEL_STORE_UNLOCK(store, cache_lock);
-
 	g_free(newibex);
 	g_free(oldibex);
 
+	if (folder)
+		camel_object_unref(folder);
+
 	return;
 
 base_failed:
@@ -355,13 +360,15 @@ summary_failed:
 	} else
 		camel_text_index_rename(newibex, oldibex);
 ibex_failed:
-	camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
-			     _("Could not rename '%s': %s"),
-			     old, strerror(errno));
+	camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+			      _("Could not rename '%s': %s"),
+			      old, g_strerror (errno));
 
-	CAMEL_STORE_UNLOCK(store, cache_lock);
 	g_free(newibex);
 	g_free(oldibex);
+
+	if (folder)
+		camel_object_unref(folder);
 }
 
 /* default implementation, only delete metadata */
@@ -369,16 +376,18 @@ static void
 delete_folder(CamelStore *store, const char *folder_name, CamelException *ex)
 {
 	CamelFolderInfo *fi;
+	CamelException lex;
+	CamelFolder *lf;
 	char *name;
 	char *str;
-
+	
 	/* remove metadata only */
 	name = g_strdup_printf("%s%s", CAMEL_LOCAL_STORE(store)->toplevel_dir, folder_name);
 	str = g_strdup_printf("%s.ev-summary", name);
 	if (unlink(str) == -1 && errno != ENOENT) {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
-				     _("Could not delete folder summary file `%s': %s"),
-				     str, strerror(errno));
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Could not delete folder summary file `%s': %s"),
+				      str, g_strerror (errno));
 		g_free(str);
 		g_free (name);
 		return;
@@ -386,25 +395,48 @@ delete_folder(CamelStore *store, const char *folder_name, CamelException *ex)
 	g_free(str);
 	str = g_strdup_printf("%s.ibex", name);
 	if (camel_text_index_remove(str) == -1 && errno != ENOENT) {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
-				     _("Could not delete folder index file `%s': %s"),
-				     str, strerror(errno));
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Could not delete folder index file `%s': %s"),
+				      str, g_strerror (errno));
 		g_free(str);
 		g_free (name);
 		return;
 	}
 	g_free(str);
-	g_free(name);
+	
+	camel_exception_init (&lex);
+	if ((lf = camel_store_get_folder (store, folder_name, 0, &lex))) {
+		camel_object_get (lf, NULL, CAMEL_OBJECT_STATE_FILE, &str, NULL);
+		camel_object_set (lf, NULL, CAMEL_OBJECT_STATE_FILE, NULL, NULL);
+		camel_object_unref (lf);
+	} else {
+		camel_exception_clear (&lex);
+	}
+	
+	if (str == NULL)
+		str = g_strdup_printf ("%s.cmeta", name);
+	
+	if (unlink (str) == -1 && errno != ENOENT) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+				      _("Could not delete folder meta file `%s': %s"),
+				      str, g_strerror (errno));
+		g_free (name);
+		g_free (str);
+		return;
+	}
+	
+	g_free (str);
+	g_free (name);
 	
 	fi = g_new0 (CamelFolderInfo, 1);
 	fi->full_name = g_strdup (folder_name);
-	fi->name = g_strdup (g_basename (folder_name));
-	fi->url = g_strdup_printf ("%s%s", CAMEL_LOCAL_STORE(store)->toplevel_dir, folder_name);
+	fi->name = g_path_get_basename (folder_name);
+	fi->url = g_strdup_printf ("%s:%s#%s", ((CamelService *) store)->url->protocol,
+				   CAMEL_LOCAL_STORE(store)->toplevel_dir, folder_name);
 	fi->unread_message_count = -1;
 	camel_folder_info_build_path(fi, '/');
-
-	camel_object_trigger_event (CAMEL_OBJECT (store),
-				    "folder_deleted", fi);
+	
+	camel_object_trigger_event (store, "folder_deleted", fi);
 	
 	camel_folder_info_free (fi);
 }
