@@ -31,11 +31,14 @@
 #include <glade/glade.h>
 #include <libgnome/gnome-i18n.h>
 
+#include "camel/camel-url.h"
 #include "vfolder-context.h"
 #include "vfolder-rule.h"
-#include "shell/evolution-shell-client.h"
+#include "mail/em-folder-tree.h"
+#include "mail/em-folder-selector.h"
+#include "mail/mail-component.h"
 
-#define d(x) x
+#define d(x) 
 
 static int validate(FilterRule *);
 static int vfolder_eq(FilterRule *fr, FilterRule *cm);
@@ -45,8 +48,6 @@ static void rule_copy (FilterRule *dest, FilterRule *src);
 /*static void build_code(FilterRule *, GString *out);*/
 static GtkWidget *get_widget(FilterRule *fr, RuleContext *f);
 
-extern EvolutionShellClient *global_shell_client;
-
 static void vfolder_rule_class_init (VfolderRuleClass *klass);
 static void vfolder_rule_init (VfolderRule *vr);
 static void vfolder_rule_finalise (GObject *obj);
@@ -391,37 +392,82 @@ select_source_with (GtkWidget *widget, struct _source_data *data)
 	filter_rule_set_source ((FilterRule *) data->vr, source);
 }
 
-static void
-source_add (GtkWidget *widget, struct _source_data *data)
+/* attempt to make a 'nice' folder name out of the raw uri */
+static char *format_source(const char *euri)
 {
-	static const char *allowed_types[] = { "mail/*", NULL };
-	GNOME_Evolution_Folder *folder;
-	GtkTreeSelection *selection;
-	GtkWidget *window;
-	GtkTreeIter iter;
-	char *uri;
+	CamelURL *url;
+	GString *out;
+	char *res, *uri;
+
+	/* This should really probably base it on the account name? */
+	uri = em_uri_to_camel(euri);
+	url = camel_url_new(uri, NULL);
+
+	/* bad uri */
+	if (url == NULL)
+		return uri;
+
+	g_free(uri);
+
+	out = g_string_new(url->protocol);
+	g_string_append_c(out, ':');
+	if (url->user && url->host) {
+		g_string_append_printf(out, "%s@%s", url->user, url->host);
+		if (url->port)
+			g_string_append_printf(out, ":%d", url->port);
+	}
+	if (url->fragment)
+		g_string_append(out, url->fragment);
+	else if (url->path)
+		g_string_append(out, url->path);
+
+	res = out->str;
+	g_string_free(out, FALSE);
+
+	return res;
+}
+
+static void
+vfr_folder_response(GtkWidget *dialog, gint button, struct _source_data *data)
+{
+	const char *uri = em_folder_selector_get_selected_uri((EMFolderSelector *)dialog);
+
+	if (button == GTK_RESPONSE_OK && uri != NULL) {
+		char *urinice, *euri;
+		GtkTreeSelection *selection;
+		GtkTreeIter iter;
 	
-	window = gtk_widget_get_toplevel (widget);
-	gtk_widget_set_sensitive (window, FALSE);
-	
-	evolution_shell_client_user_select_folder (global_shell_client, GTK_WINDOW (window),
-						   _("Select Folder"), "", allowed_types, &folder);
-	
-	gtk_widget_set_sensitive (window, TRUE);
-	
-	if (folder) {
-		uri = g_strdup (folder->physicalUri);
-		data->vr->sources = g_list_append (data->vr->sources, uri);
+		euri = em_uri_from_camel(uri);
+
+		data->vr->sources = g_list_append (data->vr->sources, euri);
 		
 		gtk_list_store_append (data->model, &iter);
-		gtk_list_store_set (data->model, &iter, 0, uri, -1);
-		selection = gtk_tree_view_get_selection (data->list);
-		gtk_tree_selection_select_iter (selection, &iter);
-		data->current = uri;
+		urinice = format_source(euri);
+		gtk_list_store_set(data->model, &iter, 0, urinice, 1, euri, -1);
+		g_free(urinice);
+		selection = gtk_tree_view_get_selection(data->list);
+		gtk_tree_selection_select_iter(selection, &iter);
+		data->current = euri;
+
+		set_sensitive(data);
 	}
+
+	gtk_widget_destroy(dialog);
+}
+
+static void
+source_add(GtkWidget *widget, struct _source_data *data)
+{
+	EMFolderTree *emft;
+	GtkWidget *dialog;
 	
-	CORBA_free (folder);
-	set_sensitive (data);
+	emft = (EMFolderTree *) em_folder_tree_new_with_model(mail_component_peek_tree_model(mail_component_peek()));
+	
+	dialog = em_folder_selector_new(emft, EM_FOLDER_SELECTOR_CAN_CREATE, _("Select Folder"), NULL);
+	gtk_window_set_transient_for((GtkWindow *)dialog, (GtkWindow *)gtk_widget_get_toplevel(widget));
+	gtk_window_set_modal((GtkWindow *)dialog, TRUE);
+	g_signal_connect(dialog, "response", G_CALLBACK(vfr_folder_response), data);
+	gtk_widget_show(dialog);
 }
 
 static void
@@ -487,10 +533,11 @@ vfolder_editor_sourcelist_new (char *widget_name, char *string1, char *string2,
 	GtkListStore *model;
 	
 	scrolled = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled), GTK_SHADOW_IN);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	
-	model = gtk_list_store_new (1, G_TYPE_STRING);
+	model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
 	table = gtk_tree_view_new_with_model ((GtkTreeModel *) model);
 	gtk_tree_view_set_headers_visible ((GtkTreeView *) table, FALSE);
 	
@@ -557,8 +604,11 @@ get_widget (FilterRule *fr, RuleContext *rc)
 	
 	source = NULL;
 	while ((source = vfolder_rule_next_source (vr, source))) {
+		char *nice = format_source(source);
+
 		gtk_list_store_append (data->model, &iter);
-		gtk_list_store_set (data->model, &iter, 0, source, -1);
+		gtk_list_store_set (data->model, &iter, 0, nice, 1, source, -1);
+		g_free(nice);
 	}
 	
 	g_signal_connect (data->list, "cursor-changed", G_CALLBACK (select_source), data);
