@@ -263,6 +263,7 @@ em_folder_selector_create_new (EMFolderTree *emft, guint32 flags, const char *ti
 	w = gtk_label_new_with_mnemonic (_("Folder _name:"));
 	gtk_box_pack_start ((GtkBox *) hbox, w, FALSE, FALSE, 6);
 	emfs->name_entry = (GtkEntry *) gtk_entry_new ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL (w), (GtkWidget *) emfs->name_entry);
 	g_signal_connect (emfs->name_entry, "changed", G_CALLBACK (emfs_create_name_changed), emfs);
 	g_signal_connect (emfs->name_entry, "activate", G_CALLBACK (emfs_create_name_activate), emfs);
 	gtk_box_pack_start ((GtkBox *) hbox, (GtkWidget *) emfs->name_entry, TRUE, FALSE, 6);
@@ -280,6 +281,11 @@ em_folder_selector_set_selected (EMFolderSelector *emfs, const char *uri)
 	em_folder_tree_set_selected (emfs->emft, uri);
 }
 
+void
+em_folder_selector_set_selected_list (EMFolderSelector *emfs, GList *list)
+{
+	em_folder_tree_set_selected_list (emfs->emft, list);
+}
 
 const char *
 em_folder_selector_get_selected_uri (EMFolderSelector *emfs)
@@ -329,6 +335,17 @@ em_folder_selector_get_selected_uri (EMFolderSelector *emfs)
 	return uri;
 }
 
+GList *
+em_folder_selector_get_selected_uris (EMFolderSelector *emfs)
+{
+	return em_folder_tree_get_selected_uris (emfs->emft);
+}
+
+GList *
+em_folder_selector_get_selected_paths (EMFolderSelector *emfs)
+{
+	return em_folder_tree_get_selected_paths (emfs->emft);
+}
 
 const char *
 em_folder_selector_get_selected_path (EMFolderSelector *emfs)
