@@ -82,6 +82,7 @@
 #include "evolution-shell-component-utils.h" /* Pixmap stuff, sigh */
 
 static void emfv_folder_changed(CamelFolder *folder, CamelFolderChangeInfo *changes, EMFolderView *emfv);
+static void emfv_message_changed(CamelFolder *folder, const char *uid, EMFolderView *emfv);
 
 static void emfv_list_message_selected(MessageList *ml, const char *uid, EMFolderView *emfv);
 static int emfv_list_right_click(ETree *tree, gint row, ETreePath path, gint col, GdkEvent *event, EMFolderView *emfv);
@@ -112,10 +113,8 @@ static const EMFolderViewEnable emfv_enable_map[];
 struct _EMFolderViewPrivate {
 	guint seen_id;
 	guint setting_notify_id;
-
-	char *displayed_uid;	/* only used to stop re-loads, don't use it to represent any selection state */
 	
-	CamelObjectHookID folder_changed_id;
+	CamelObjectHookID folder_changed_id, message_changed_id;
 
 	GtkWidget *invisible;
 	char *selection_uri;
@@ -188,6 +187,8 @@ emfv_finalise(GObject *o)
 	if (emfv->folder) {
 		if (p->folder_changed_id)
 			camel_object_remove_event(emfv->folder, p->folder_changed_id);
+		if (p->message_changed_id)
+			camel_object_remove_event(emfv->folder, p->message_changed_id);
 		camel_object_unref(emfv->folder);
 		g_free(emfv->folder_uri);
 	}
@@ -359,14 +360,16 @@ emfv_set_folder(EMFolderView *emfv, CamelFolder *folder, const char *uri)
 	if (folder != emfv->folder) {
 		if (emfv->folder) {
 			mail_sync_folder (emfv->folder, NULL, NULL);
-			if (emfv->priv->folder_changed_id)
-				camel_object_remove_event(emfv->folder, emfv->priv->folder_changed_id);
+			camel_object_remove_event(emfv->folder, emfv->priv->folder_changed_id);
+			camel_object_remove_event(emfv->folder, emfv->priv->message_changed_id);
 			camel_object_unref(emfv->folder);
 		}
 		emfv->folder = folder;
 		if (folder) {
 			emfv->priv->folder_changed_id = camel_object_hook_event(folder, "folder_changed",
 										(CamelObjectEventHookFunc)emfv_folder_changed, emfv);
+			emfv->priv->message_changed_id = camel_object_hook_event(folder, "message_changed",
+										 (CamelObjectEventHookFunc)emfv_message_changed, emfv);
 			camel_object_ref(folder);
 		}
 	}
@@ -386,7 +389,7 @@ static void
 emfv_set_folder_uri(EMFolderView *emfv, const char *uri)
 {
 	if (emfv->preview)
-		em_format_format((EMFormat *)emfv->preview, NULL);
+		em_format_format((EMFormat *)emfv->preview, NULL, NULL, NULL);
 	
 	mail_get_folder(uri, 0, emfv_got_folder, emfv, mail_thread_queued);
 }
@@ -395,6 +398,8 @@ static void
 emfv_set_message(EMFolderView *emfv, const char *uid)
 {
 	message_list_select_uid(emfv->list, uid);
+	/* force an update, since we may not get an updated event if we select the same uid */
+	emfv_list_message_selected(emfv->list, uid, emfv);
 }
 
 /* ********************************************************************** */
@@ -1597,7 +1602,7 @@ int em_folder_view_print(EMFolderView *emfv, int preview)
 	EMFormatHTMLPrint *print;
 	GnomePrintConfig *config = NULL;
 	int res;
-	struct _CamelMedium *msg;
+	struct _CamelMimeMessage *msg;
 
 	/* FIXME: need to load the message first */
 	if (!emfv->preview_active)
@@ -1630,7 +1635,7 @@ int em_folder_view_print(EMFolderView *emfv, int preview)
 
 	print = em_format_html_print_new();
 	em_format_set_session((EMFormat *)print, ((EMFormat *)emfv->preview)->session);
-	res = em_format_html_print_print(print, msg, (EMFormatHTML *)emfv->preview, config, preview);
+	res = em_format_html_print_print(print, (EMFormatHTML *)emfv->preview, config, preview);
 	g_object_unref(print);
 	if (config)
 		g_object_unref(config);
@@ -1701,7 +1706,7 @@ emfv_list_done_message_selected(CamelFolder *folder, const char *uid, CamelMimeM
 {
 	EMFolderView *emfv = data;
 
-	em_format_format((EMFormat *)emfv->preview, (struct _CamelMedium *)msg);
+	em_format_format((EMFormat *)emfv->preview, folder, uid, msg);
 	
 	if (emfv->priv->seen_id)
 		g_source_remove(emfv->priv->seen_id);
@@ -1729,15 +1734,15 @@ emfv_list_message_selected(MessageList *ml, const char *uid, EMFolderView *emfv)
 
 	if (emfv->preview_active) {
 		if (uid) {
-			if (emfv->priv->displayed_uid == NULL || strcmp(emfv->priv->displayed_uid, uid) != 0) {
-				g_free(emfv->priv->displayed_uid);
-				emfv->priv->displayed_uid = g_strdup(uid);
+			if (emfv->displayed_uid == NULL || strcmp(emfv->displayed_uid, uid) != 0) {
+				g_free(emfv->displayed_uid);
+				emfv->displayed_uid = g_strdup(uid);
 				mail_get_message(emfv->folder, uid, emfv_list_done_message_selected, emfv, mail_thread_new);
 			}
 		} else {
-			g_free(emfv->priv->displayed_uid);
-			emfv->priv->displayed_uid = NULL;
-			em_format_format((EMFormat *)emfv->preview, NULL);
+			g_free(emfv->displayed_uid);
+			emfv->displayed_uid = NULL;
+			em_format_format((EMFormat *)emfv->preview, NULL, NULL, NULL);
 		}
 	}
 
@@ -1929,6 +1934,13 @@ emfv_folder_changed(CamelFolder *folder, CamelFolderChangeInfo *changes, EMFolde
 	mail_async_event_emit(emfv->async, MAIL_ASYNC_GUI, (MailAsyncFunc)emfv_gui_folder_changed, folder, NULL, emfv);
 }
 
+static void
+emfv_message_changed(CamelFolder *folder, const char *uid, EMFolderView *emfv)
+{
+	g_object_ref(emfv);
+	mail_async_event_emit(emfv->async, MAIL_ASYNC_GUI, (MailAsyncFunc)emfv_gui_folder_changed, folder, NULL, emfv);
+}
+
 /* keep these two tables in sync */
 enum {
 	EMFV_ANIMATE_IMAGES = 1,
@@ -2047,7 +2059,7 @@ emfv_setting_notify(GConfClient *gconf, guint cnxn_id, GConfEntry *entry, EMFold
 			em_format_default_headers(emf);
 		/* force a redraw */
 		if (emf->message)
-			em_format_format_clone(emf, emf->message, emf);
+			em_format_redraw(emf);
 		break; }
 	}
 }
@@ -2109,9 +2121,19 @@ emfv_on_url_cb (GObject *emitter, const char *url, EMFolderView *emfv)
 	char *nice_url = NULL;
 
 	if (url) {
-		if (strncmp (url, "mailto:", 7) == 0)
-			nice_url = g_strdup_printf (_("Click to mail %s"), url + 7);
-		else
+		if (strncmp (url, "mailto:", 7) == 0) {
+			CamelInternetAddress *cia = camel_internet_address_new();
+			CamelURL *curl;
+			char *addr;
+
+			curl = camel_url_new(url, NULL);
+			camel_address_decode((CamelAddress *)cia, curl->path);
+			addr = camel_address_format((CamelAddress *)cia);
+			nice_url = g_strdup_printf (_("Click to mail %s"), addr&&addr[0]?addr:(url + 7));
+			g_free(addr);
+			camel_url_free(curl);
+			camel_object_unref(cia);
+		} else
 			nice_url = g_strdup_printf (_("Click to open %s"), url);
 	}
 	
