@@ -77,7 +77,7 @@
 #define EFH_TABLE_OPEN "<table>"
 
 struct _EMFormatHTMLPrivate {
-	struct _CamelMedium *last_part;	/* not reffed, DO NOT dereference */
+	struct _CamelMimeMessage *last_part;	/* not reffed, DO NOT dereference */
 	volatile int format_id;		/* format thread id */
 	guint format_timeout_id;
 	struct _format_msg *format_timeout_msg;
@@ -93,7 +93,7 @@ static void efh_url_requested(GtkHTML *html, const char *url, GtkHTMLStream *han
 static gboolean efh_object_requested(GtkHTML *html, GtkHTMLEmbedded *eb, EMFormatHTML *efh);
 static void efh_gtkhtml_destroy(GtkHTML *html, EMFormatHTML *efh);
 
-static void efh_format_clone(EMFormat *, CamelMedium *, EMFormat *);
+static void efh_format_clone(EMFormat *emf, CamelFolder *folder, const char *uid, CamelMimeMessage *msg, EMFormat *emfsource);
 static void efh_format_error(EMFormat *emf, CamelStream *stream, const char *txt);
 static void efh_format_message(EMFormat *, CamelStream *, CamelMedium *);
 static void efh_format_source(EMFormat *, CamelStream *, CamelMimePart *);
@@ -231,8 +231,10 @@ em_format_html_get_type(void)
 		path = alloca(strlen(base_directory)+16);
 		sprintf(path, "%s/cache", base_directory);
 		emfh_http_cache = camel_data_cache_new(path, 0, NULL);
-		camel_data_cache_set_expire_age(emfh_http_cache, 24*60*60);
-		camel_data_cache_set_expire_access(emfh_http_cache, 2*60*60);
+		if (emfh_http_cache) {
+			camel_data_cache_set_expire_age(emfh_http_cache, 24*60*60);
+			camel_data_cache_set_expire_access(emfh_http_cache, 2*60*60);
+		}
 	}
 
 	return type;
@@ -256,7 +258,7 @@ void em_format_html_load_http(EMFormatHTML *emfh)
 	/* This will remain set while we're still rendering the same message, then it wont be */
 	emfh->load_http_now = TRUE;
 	d(printf("redrawing with images forced on\n"));
-	em_format_format_clone((EMFormat *)emfh, emfh->format.message, (EMFormat *)emfh);
+	em_format_redraw((EMFormat *)emfh);
 }
 
 void
@@ -264,7 +266,7 @@ em_format_html_set_load_http(EMFormatHTML *emfh, int state)
 {
 	if (emfh->load_http ^ state) {
 		emfh->load_http = state;
-		em_format_format_clone((EMFormat *)emfh, emfh->format.message, (EMFormat *)emfh);
+		em_format_redraw((EMFormat *)emfh);
 	}
 }
 
@@ -274,7 +276,7 @@ em_format_html_set_mark_citations(EMFormatHTML *emfh, int state, guint32 citatio
 	if (emfh->mark_citations ^ state || emfh->citation_colour != citation_colour) {
 		emfh->mark_citations = state;
 		emfh->citation_colour = citation_colour;
-		em_format_format_clone((EMFormat *)emfh, emfh->format.message, (EMFormat *)emfh);
+		em_format_redraw((EMFormat *)emfh);
 	}
 }
 
@@ -283,7 +285,7 @@ em_format_html_set_xmailer_mask(EMFormatHTML *emfh, unsigned int xmailer_mask)
 {
 	if (emfh->xmailer_mask ^ xmailer_mask) {
 		emfh->xmailer_mask = xmailer_mask;
-		em_format_format_clone((EMFormat *)emfh, emfh->format.message, (EMFormat *)emfh);
+		em_format_redraw((EMFormat *)emfh);
 	}
 }
 
@@ -325,13 +327,10 @@ em_format_html_add_pobject(EMFormatHTML *efh, size_t size, const char *classid,
 	g_assert(size >= sizeof(EMFormatHTMLPObject));
 
 	pobj = g_malloc0(size);
-	if (classid) {
+	if (classid)
 		pobj->classid = g_strdup(classid);
-	} else {
-		static unsigned int uriid = 0;
-
-		pobj->classid = g_strdup_printf("e-object:///%u", uriid++);
-	}
+	else
+		pobj->classid = g_strdup_printf("e-object:///%s", ((EMFormat *)efh)->part_id->str);
 
 	pobj->format = efh;
 	pobj->func = func;
@@ -582,6 +581,7 @@ static void
 efh_output_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, CamelCipherValidity *valid)
 {
 	CamelCipherValidity *save = ((EMFormatHTML *)emf)->valid_parent;
+	int len;
 
 	/* Note: this same logic is in efhd_output_secure */
 	if (((EMFormatHTML *)emf)->valid == NULL) {
@@ -592,7 +592,12 @@ efh_output_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, Camel
 	}
 
 	((EMFormatHTML *)emf)->valid_parent = valid;
+
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".signed");
 	em_format_part(emf, stream, part);
+	g_string_truncate(emf->part_id, len);
+
 	((EMFormatHTML *)emf)->valid_parent = save;
 
 	if (((EMFormatHTML *)emf)->valid == valid
@@ -604,7 +609,7 @@ efh_output_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, Camel
 		camel_stream_printf(stream, "<table border=0 width=\"100%%\" cellpadding=3 cellspacing=0 bgcolor=%s><tr>",
 				    valid->sign.status == CAMEL_CIPHER_VALIDITY_SIGN_GOOD?"#88bb88":"#bb8888");
 
-		classid = g_strdup_printf("smime:///em-format-html/%p/icon/signed", part);
+		classid = g_strdup_printf("smime:///em-format-html/%s/icon/signed", emf->part_id->str);
 		camel_stream_printf(stream, "<td valign=\"top\"><img src=\"%s\"></td><td valign=\"top\" width=\"100%%\">", classid);
 		iconpart = em_format_html_file_part((EMFormatHTML *)emf, "image/png",
 						    EVOLUTION_ICONSDIR, smime_sign_table[valid->sign.status].icon);
@@ -659,10 +664,11 @@ efh_text_plain(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFo
 	CamelStreamFilter *filtered_stream;
 	CamelMimeFilter *html_filter;
 	CamelMultipart *mp;
+	CamelDataWrapper *dw;
 	CamelContentType *type;
 	const char *format;
 	guint32 rgb = 0x737373, flags;
-	int i, count;
+	int i, count, len;
 
 	camel_stream_printf (stream,
 			     "<table bgcolor=\"#%06x\" cellspacing=0 cellpadding=1 width=100%%><tr><td>\n"
@@ -672,10 +678,11 @@ efh_text_plain(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFo
 
 	flags = efh->text_html_flags;
 	
+	dw = camel_medium_get_content_object((CamelMedium *)part);
+
 	/* Check for RFC 2646 flowed text. */
-	type = camel_mime_part_get_content_type(part);
-	if (camel_content_type_is (type, "text", "plain")
-	    && (format = camel_content_type_param (type, "format"))
+	if (camel_content_type_is(dw->mime_type, "text", "plain")
+	    && (format = camel_content_type_param(dw->mime_type, "format"))
 	    && !g_ascii_strcasecmp(format, "flowed"))
 		flags |= CAMEL_MIME_FILTER_TOHTML_FORMAT_FLOWED;
 
@@ -698,9 +705,9 @@ efh_text_plain(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFo
 		null = camel_stream_null_new();
 		filtered_stream = camel_stream_filter_new_with_stream(null);
 		camel_object_unref(null);
-		inline_filter = em_inline_filter_new(camel_mime_part_get_encoding(part), ((CamelDataWrapper *)part)->mime_type);
+		inline_filter = em_inline_filter_new(camel_mime_part_get_encoding(part), dw->mime_type);
 		camel_stream_filter_add(filtered_stream, (CamelMimeFilter *)inline_filter);
-		camel_data_wrapper_write_to_stream(camel_medium_get_content_object((CamelMedium *)part), (CamelStream *)filtered_stream);
+		camel_data_wrapper_write_to_stream(dw, (CamelStream *)filtered_stream);
 		camel_stream_close((CamelStream *)filtered_stream);
 		camel_object_unref(filtered_stream);
 		mp = em_inline_filter_get_multipart(inline_filter);
@@ -715,6 +722,7 @@ efh_text_plain(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFo
 
 	/* We handle our made-up multipart here, so we don't recursively call ourselves */
 
+	len = ((EMFormat *)efh)->part_id->len;
 	count = camel_multipart_get_number(mp);
 	for (i=0;i<count;i++) {
 		CamelMimePart *newpart = camel_multipart_get_part(mp, i);
@@ -726,7 +734,9 @@ efh_text_plain(EMFormatHTML *efh, CamelStream *stream, CamelMimePart *part, EMFo
 			camel_stream_flush((CamelStream *)filtered_stream);
 			camel_stream_write_string(stream, "</tt>\n");
 		} else {
+			g_string_append_printf(((EMFormat *)efh)->part_id, ".inline.%d", i);
 			em_format_part((EMFormat *)efh, stream, newpart);
+			g_string_truncate(((EMFormat *)efh)->part_id, len);
 		}
 	}
 
@@ -921,11 +931,13 @@ emfh_multipart_related_check(struct _EMFormatHTMLJob *job, int cancelled)
 {
 	struct _EMFormatPURITree *ptree;
 	EMFormatPURI *puri, *purin;
+	char *oldpartid;
 
 	if (cancelled)
 		return;
 
 	d(printf(" running multipart/related check task\n"));
+	oldpartid = g_strdup(((EMFormat *)job->format)->part_id->str);
 
 	ptree = job->puri_level;
 	puri = (EMFormatPURI *)ptree->uri_list.head;
@@ -933,13 +945,18 @@ emfh_multipart_related_check(struct _EMFormatHTMLJob *job, int cancelled)
 	while (purin) {
 		if (puri->use_count == 0) {
 			d(printf("part '%s' '%s' used '%d'\n", puri->uri?puri->uri:"", puri->cid, puri->use_count));
-			if (puri->func == emfh_write_related)
+			if (puri->func == emfh_write_related) {
+				g_string_printf(((EMFormat *)job->format)->part_id, puri->part_id);
 				em_format_part((EMFormat *)job->format, (CamelStream *)job->stream, puri->part);
+			}
 			/* else it was probably added by a previous format this loop */
 		}
 		puri = purin;
 		purin = purin->next;
 	}
+
+	g_string_printf(((EMFormat *)job->format)->part_id, "%s", oldpartid);
+	g_free(oldpartid);
 }
 
 /* RFC 2387 */
@@ -950,7 +967,7 @@ efh_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	CamelMimePart *body_part, *display_part = NULL;
 	CamelContentType *content_type;
 	const char *location, *start;
-	int i, nparts;
+	int i, nparts, partidlen, displayid = 0;
 	CamelURL *base_save = NULL;
 	EMFormatPURI *puri;
 	struct _EMFormatHTMLJob *job;
@@ -977,6 +994,7 @@ efh_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 			
 			if (cid && !strncmp(cid, start, len) && strlen(cid) == len) {
 				display_part = body_part;
+				displayid = i;
 				break;
 			}
 		}
@@ -998,16 +1016,22 @@ efh_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	}
 	em_format_push_level(emf);
 
+	partidlen = emf->part_id->len;
+
 	/* queue up the parts for possible inclusion */
 	for (i = 0; i < nparts; i++) {
 		body_part = camel_multipart_get_part(mp, i);
 		if (body_part != display_part) {
+			g_string_append_printf(emf->part_id, "related.%d", i);
 			puri = em_format_add_puri(emf, sizeof(EMFormatPURI), NULL, body_part, emfh_write_related);
+			g_string_truncate(emf->part_id, partidlen);
 			d(printf(" part '%s' '%s' added\n", puri->uri?puri->uri:"", puri->cid));
 		}
 	}
 	
+	g_string_append_printf(emf->part_id, "related.%d", displayid);
 	em_format_part(emf, stream, display_part);
+	g_string_truncate(emf->part_id, partidlen);
 	camel_stream_flush(stream);
 
 	/* queue a job to check for un-referenced parts to add as attachments */
@@ -1110,10 +1134,9 @@ static EMFormatHandler type_builtin_table[] = {
 	{ "image/x-xpixmap", (EMFormatFunc)efh_image },
 	{ "text/enriched", (EMFormatFunc)efh_text_enriched },
 	{ "text/plain", (EMFormatFunc)efh_text_plain },
-	{ "text/x-patch", (EMFormatFunc)efh_text_plain },
 	{ "text/html", (EMFormatFunc)efh_text_html },
 	{ "text/richtext", (EMFormatFunc)efh_text_enriched },
-	/*{ "text/ *", (EMFormatFunc)efh_text_plain },*/
+	{ "text/*", (EMFormatFunc)efh_text_plain },
 	{ "message/external-body", (EMFormatFunc)efh_message_external },
 	{ "multipart/signed", (EMFormatFunc)efh_multipart_signed },
 	{ "multipart/related", (EMFormatFunc)efh_multipart_related },
@@ -1144,7 +1167,9 @@ struct _format_msg {
 	EMFormatHTML *format;
 	EMFormat *format_source;
 	EMHTMLStream *estream;
-	CamelMedium *message;
+	CamelFolder *folder;
+	char *uid;
+	CamelMimeMessage *message;
 };
 
 static char *efh_format_desc(struct _mail_msg *mm, int done)
@@ -1172,10 +1197,12 @@ static void efh_format_do(struct _mail_msg *mm)
 
 	/* <insert top-header stuff here> */
 
-	if (((EMFormat *)m->format)->mode == EM_FORMAT_SOURCE)
+	if (((EMFormat *)m->format)->mode == EM_FORMAT_SOURCE) {
 		em_format_format_source((EMFormat *)m->format, (CamelStream *)m->estream, (CamelMimePart *)m->message);
-	else
-		em_format_format_message((EMFormat *)m->format, (CamelStream *)m->estream, m->message);
+	} else {
+		em_format_format_prefix((EMFormat *)m->format, (CamelStream *)m->estream);
+		em_format_format_message((EMFormat *)m->format, (CamelStream *)m->estream, (CamelMedium *)m->message);
+	}
 
 	camel_stream_write_string((CamelStream *)m->estream, "</body>\n</html>\n");
 	camel_stream_close((CamelStream *)m->estream);
@@ -1239,6 +1266,9 @@ static void efh_format_free(struct _mail_msg *mm)
 		camel_stream_close((CamelStream *)m->estream);
 		camel_object_unref(m->estream);
 	}
+	if (m->folder)
+		camel_object_unref(m->folder);
+	g_free(m->uid);
 	if (m->message)
 		camel_object_unref(m->message);
 	if (m->format_source)
@@ -1275,7 +1305,7 @@ efh_format_timeout(struct _format_msg *m)
 	d(printf(" ready to go, firing off format thread\n"));
 
 	/* call super-class to kick it off */
-	efh_parent->format_clone((EMFormat *)efh, m->message, m->format_source);
+	efh_parent->format_clone((EMFormat *)efh, m->folder, m->uid, m->message, m->format_source);
 	em_format_html_clear_pobject(m->format);
 
 	if (efh->valid) {
@@ -1319,7 +1349,7 @@ efh_format_timeout(struct _format_msg *m)
 	return FALSE;
 }
 
-static void efh_format_clone(EMFormat *emf, CamelMedium *part, EMFormat *emfsource)
+static void efh_format_clone(EMFormat *emf, CamelFolder *folder, const char *uid, CamelMimeMessage *msg, EMFormat *emfsource)
 {
 	EMFormatHTML *efh = (EMFormatHTML *)emf;
 	struct _format_msg *m;
@@ -1344,9 +1374,13 @@ static void efh_format_clone(EMFormat *emf, CamelMedium *part, EMFormat *emfsour
 	m->format_source = emfsource;
 	if (emfsource)
 		g_object_ref(emfsource);
-	m->message = part;
-	if (part)
-		camel_object_ref(part);
+	m->folder = folder;
+	if (folder)
+		camel_object_ref(folder);
+	m->uid = g_strdup(uid);
+	m->message = msg;
+	if (msg)
+		camel_object_ref(msg);
 
 	if (efh->priv->format_id == -1) {
 		d(printf(" idle, forcing format\n"));
@@ -1420,6 +1454,59 @@ static char *i18n_hdrs[] = {
 };
 #endif
 
+static void
+efh_format_address (GString *out, struct _camel_header_address *a)
+{
+	guint32 flags = CAMEL_MIME_FILTER_TOHTML_CONVERT_SPACES;
+	char *name, *mailto, *addr;
+	
+	while (a) {
+		if (a->name)
+			name = camel_text_to_html (a->name, flags, 0);
+		else
+			name = NULL;
+		
+		switch (a->type) {
+		case CAMEL_HEADER_ADDRESS_NAME:
+			if (name && *name) {
+				char *real, *mailaddr;
+
+				g_string_append_printf (out, "%s &lt;", name);
+				/* rfc2368 for mailto syntax and url encoding extras */
+				real = camel_header_encode_phrase(a->name);
+				mailaddr = g_strdup_printf("%s <%s>", real, a->v.addr);
+				g_free(real);
+				mailto = camel_url_encode(mailaddr, "?=&()");
+				g_free(mailaddr);
+			} else {
+				mailto = camel_url_encode (a->v.addr, "?=&()");
+			}
+			addr = camel_text_to_html (a->v.addr, flags, 0);
+			g_string_append_printf (out, "<a href=\"mailto:%s\">%s</a>", mailto, addr);
+			g_free (mailto);
+			g_free (addr);
+			
+			if (name && *name)
+				g_string_append (out, "&gt;");
+			break;
+		case CAMEL_HEADER_ADDRESS_GROUP:
+			g_string_append_printf (out, "%s: ", name);
+			efh_format_address (out, a->v.members);
+			g_string_append_printf (out, ";");
+			break;
+		default:
+			g_warning ("Invalid address type");
+			break;
+		}
+		
+		g_free (name);
+		
+		a = a->next;
+		if (a)
+			g_string_append (out, ", ");
+	}
+}
+
 static void
 efh_format_header(EMFormat *emf, CamelStream *stream, CamelMedium *part, const char *namein, guint32 flags, const char *charset)
 {
@@ -1442,6 +1529,7 @@ efh_format_header(EMFormat *emf, CamelStream *stream, CamelMedium *part, const c
 	
 	if (addrspec) {
 		struct _camel_header_address *addrs;
+		GString *html;
 		
 		if (!(txt = camel_medium_get_header (part, name)))
 			return;
@@ -1462,11 +1550,13 @@ efh_format_header(EMFormat *emf, CamelStream *stream, CamelMedium *part, const c
 		
 		label = _(name);
 		
-		if ((value = camel_header_address_list_format (addrs)))
-			txt = value;
-		
-		flags |= EM_FORMAT_HEADER_BOLD;
+		html = g_string_new ("");
+		efh_format_address (html, addrs);
 		camel_header_address_unref (addrs);
+		txt = value = html->str;
+		g_string_free (html, FALSE);
+		
+		flags |= EM_FORMAT_HEADER_BOLD | EM_FORMAT_HTML_HEADER_HTML;
 	} else if (!strcmp (name, "subject")) {
 		txt = camel_mime_message_get_subject (msg);
 		label = _("Subject");
@@ -1582,7 +1672,7 @@ static void efh_format_message(EMFormat *emf, CamelStream *stream, CamelMedium *
 	efh->valid = NULL;
 	efh->valid_parent = NULL;
 
-	if (emf->message != part)
+	if (emf->message != (CamelMimeMessage *)part)
 		camel_stream_printf(stream, "<blockquote>\n");
 
 	if (!efh->hide_headers)
@@ -1591,7 +1681,7 @@ static void efh_format_message(EMFormat *emf, CamelStream *stream, CamelMedium *
 	camel_stream_printf(stream, "<table height=6><tr><td><a></a></td></tr></table>\n");
 	em_format_part(emf, stream, (CamelMimePart *)part);
 
-	if (emf->message != part)
+	if (emf->message != (CamelMimeMessage *)part)
 		camel_stream_printf(stream, "</blockquote>\n");
 
 	camel_cipher_validity_free(efh->valid);
