@@ -57,7 +57,9 @@
 static void emf_builtin_init(EMFormatClass *);
 static const char *emf_snoop_part(CamelMimePart *part);
 
-static void emf_format_clone(EMFormat *emf, CamelMedium *msg, EMFormat *emfsource);
+static const EMFormatHandler *emf_find_handler(EMFormat *emf, const char *mime_type);
+static void emf_format_clone(EMFormat *emf, CamelFolder *folder, const char *uid, CamelMimeMessage *msg, EMFormat *emfsource);
+static void emf_format_prefix(EMFormat *emf, CamelStream *stream);
 static gboolean emf_busy(EMFormat *emf);
 
 enum {
@@ -76,6 +78,7 @@ emf_init(GObject *o)
 	emf->inline_table = g_hash_table_new(NULL, NULL);
 	e_dlist_init(&emf->header_list);
 	em_format_default_headers(emf);
+	emf->part_id = g_string_new("");
 }
 
 static void
@@ -91,7 +94,8 @@ emf_finalise(GObject *o)
 
 	em_format_clear_headers(emf);
 	g_free(emf->charset);
-	
+	g_string_free(emf->part_id, TRUE);
+
 	/* FIXME: check pending jobs */
 	
 	((GObjectClass *)emf_parent)->finalize(o);
@@ -111,7 +115,9 @@ emf_class_init(GObjectClass *klass)
 	emf_builtin_init((EMFormatClass *)klass);
 
 	klass->finalize = emf_finalise;
+	((EMFormatClass *)klass)->find_handler = emf_find_handler;
 	((EMFormatClass *)klass)->format_clone = emf_format_clone;
+	((EMFormatClass *)klass)->format_prefix = emf_format_prefix;
 	((EMFormatClass *)klass)->busy = emf_busy;
 
 	emf_signals[EMF_COMPLETE] =
@@ -184,7 +190,6 @@ em_format_class_remove_handler (EMFormatClass *emfc, const char *mime_type)
 	g_hash_table_remove (emfc->type_handlers, mime_type);
 }
 
-
 /**
  * em_format_find_handler:
  * @emf: 
@@ -194,8 +199,8 @@ em_format_class_remove_handler (EMFormatClass *emfc, const char *mime_type)
  * 
  * Return value: NULL if no handler is available.
  **/
-const EMFormatHandler *
-em_format_find_handler(EMFormat *emf, const char *mime_type)
+static const EMFormatHandler *
+emf_find_handler(EMFormat *emf, const char *mime_type)
 {
 	EMFormatClass *emfc = (EMFormatClass *)G_OBJECT_GET_CLASS(emf);
 
@@ -261,7 +266,6 @@ em_format_add_puri(EMFormat *emf, size_t size, const char *cid, CamelMimePart *p
 {
 	EMFormatPURI *puri;
 	const char *tmp;
-	static unsigned int uriid;
 
 	g_assert(size >= sizeof(*puri));
 	puri = g_malloc0(size);
@@ -270,6 +274,7 @@ em_format_add_puri(EMFormat *emf, size_t size, const char *cid, CamelMimePart *p
 	puri->func = func;
 	puri->use_count = 0;
 	puri->cid = g_strdup(cid);
+	puri->part_id = g_strdup(emf->part_id->str);
 
 	if (part) {
 		camel_object_ref(part);
@@ -281,7 +286,7 @@ em_format_add_puri(EMFormat *emf, size_t size, const char *cid, CamelMimePart *p
 		if (tmp)
 			puri->cid = g_strdup_printf("cid:%s", tmp);
 		else
-			puri->cid = g_strdup_printf("em-no-cid-%u", uriid++);
+			puri->cid = g_strdup_printf("em-no-cid:%s", emf->part_id->str);
 
 		d(printf("built cid '%s'\n", puri->cid));
 
@@ -420,6 +425,7 @@ emf_clear_puri_node(struct _EMFormatPURITree *node)
 		while (pn) {
 			g_free(pw->uri);
 			g_free(pw->cid);
+			g_free(pw->part_id);
 			if (pw->part)
 				camel_object_unref(pw->part);
 			g_free(pw);
@@ -514,7 +520,7 @@ emf_clone_inlines(void *key, void *val, void *data)
 }
 
 static void
-emf_format_clone(EMFormat *emf, CamelMedium *msg, EMFormat *emfsource)
+emf_format_clone(EMFormat *emf, CamelFolder *folder, const char *uid, CamelMimeMessage *msg, EMFormat *emfsource)
 {
 	em_format_clear_puri_tree(emf);
 
@@ -531,6 +537,20 @@ emf_format_clone(EMFormat *emf, CamelMedium *msg, EMFormat *emfsource)
 		}
 	}
 
+	/* what a mess */
+	if (folder != emf->folder) {
+		if (emf->folder)
+			camel_object_unref(emf->folder);
+		if (folder)
+			camel_object_ref(folder);
+		emf->folder = folder;
+	}
+
+	if (uid != emf->uid) {
+		g_free(emf->uid);
+		emf->uid = g_strdup(uid);
+	}
+
 	if (msg != emf->message) {
 		if (emf->message)
 			camel_object_unref(emf->message);
@@ -538,6 +558,19 @@ emf_format_clone(EMFormat *emf, CamelMedium *msg, EMFormat *emfsource)
 			camel_object_ref(msg);
 		emf->message = msg;
 	}
+
+	g_string_truncate(emf->part_id, 0);
+	if (folder != NULL)
+		/* TODO build some string based on the folder name/location? */
+		g_string_append_printf(emf->part_id, ".%p", folder);
+	if (uid != NULL)
+		g_string_append_printf(emf->part_id, ".%s", uid);
+}
+
+static void
+emf_format_prefix(EMFormat *emf, CamelStream *stream)
+{
+	/* NOOP */
 }
 
 static gboolean
@@ -549,7 +582,9 @@ emf_busy(EMFormat *emf)
 /**
  * em_format_format_clone:
  * @emf: Mail formatter.
- * @msg: Mail message.
+ * @folder: Camel Folder.
+ * @uid: Uid of message.
+ * @msg: Camel Message.
  * @emfsource: Used as a basis for user-altered layout, e.g. inline viewed
  * attachments.
  * 
@@ -599,7 +634,7 @@ em_format_set_mode(EMFormat *emf, em_format_mode_t type)
 
 	/* force redraw if type changed afterwards */
 	if (emf->message)
-		em_format_format_clone(emf, emf->message, emf);
+		em_format_redraw(emf);
 }
 
 /**
@@ -622,10 +657,9 @@ em_format_set_charset(EMFormat *emf, const char *charset)
 	emf->charset = g_strdup(charset);
 
 	if (emf->message)
-		em_format_format_clone(emf, emf->message, emf);
+		em_format_redraw(emf);
 }
 
-
 /**
  * em_format_set_default_charset:
  * @emf: 
@@ -647,7 +681,7 @@ em_format_set_default_charset(EMFormat *emf, const char *charset)
 	emf->default_charset = g_strdup(charset);
 
 	if (emf->message && emf->charset == NULL)
-		em_format_format_clone(emf, emf->message, emf);
+		em_format_redraw(emf);
 }
 
 /**
@@ -828,13 +862,13 @@ em_format_format_text(EMFormat *emf, CamelStream *stream, CamelDataWrapper *dw)
 	CamelStreamFilter *filter_stream;
 	CamelMimeFilterCharset *filter;
 	const char *charset = NULL;
+	CamelMimeFilterWindows *windows = NULL;
 	
 	if (emf->charset) {
 		charset = emf->charset;
 	} else if (dw->mime_type
 		   && (charset = camel_content_type_param (dw->mime_type, "charset"))
 		   && g_ascii_strncasecmp(charset, "iso-8859-", 9) == 0) {
-		CamelMimeFilterWindows *windows;
 		CamelStream *null;
 
 		/* Since a few Windows mailers like to claim they sent
@@ -854,7 +888,6 @@ em_format_format_text(EMFormat *emf, CamelStream *stream, CamelDataWrapper *dw)
 		camel_object_unref(filter_stream);
 		
 		charset = camel_mime_filter_windows_real_charset (windows);
-		camel_object_unref(windows);
 	} else if (charset == NULL) {
 		charset = emf->default_charset;
 	}
@@ -869,6 +902,9 @@ em_format_format_text(EMFormat *emf, CamelStream *stream, CamelDataWrapper *dw)
 	camel_data_wrapper_decode_to_stream(dw, (CamelStream *)filter_stream);
 	camel_stream_flush((CamelStream *)filter_stream);
 	camel_object_unref(filter_stream);
+
+	if (windows)
+		camel_object_unref(windows);
 }
 
 /**
@@ -957,14 +993,18 @@ static void
 emf_multipart_appledouble(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)part);
-	
+	int len;
+
 	if (!CAMEL_IS_MULTIPART(mp)) {
 		em_format_format_source(emf, stream, part);
 		return;
 	}
 
 	/* try the data fork for something useful, doubtful but who knows */
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".appledouble.1");
 	em_format_part(emf, stream, camel_multipart_get_part(mp, 1));
+	g_string_truncate(emf->part_id, len);
 }
 
 /* RFC ??? */
@@ -972,17 +1012,20 @@ static void
 emf_multipart_mixed(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)part);
-	int i, nparts;
+	int i, nparts, len;
 
 	if (!CAMEL_IS_MULTIPART(mp)) {
 		em_format_format_source(emf, stream, part);
 		return;
 	}
-	
+
+	len = emf->part_id->len;
 	nparts = camel_multipart_get_number(mp);	
 	for (i = 0; i < nparts; i++) {
 		part = camel_multipart_get_part(mp, i);
+		g_string_append_printf(emf->part_id, ".mixed.%d", i);
 		em_format_part(emf, stream, part);
+		g_string_truncate(emf->part_id, len);
 	}
 }
 
@@ -991,7 +1034,7 @@ static void
 emf_multipart_alternative(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)part);
-	int i, nparts;
+	int i, nparts, bestid;
 	CamelMimePart *best = NULL;
 
 	if (!CAMEL_IS_MULTIPART(mp)) {
@@ -1012,15 +1055,21 @@ emf_multipart_alternative(EMFormat *emf, CamelStream *stream, CamelMimePart *par
 		  return part;*/
 
 		if (em_format_find_handler(emf, mime_type)
-		    || (best == NULL && em_format_fallback_handler(emf, mime_type)))
+		    || (best == NULL && em_format_fallback_handler(emf, mime_type))) {
 			best = part;
+			bestid = i;
+		}
 
 		g_free(mime_type);
 	}
 
-	if (best)
+	if (best) {
+		int len = emf->part_id->len;
+
+		g_string_append_printf(emf->part_id, ".alternative.%d", bestid);
 		em_format_part(emf, stream, best);
-	else
+		g_string_truncate(emf->part_id, len);
+	} else
 		emf_multipart_mixed(emf, stream, part, info);
 }
 
@@ -1032,6 +1081,7 @@ emf_multipart_encrypted(EMFormat *emf, CamelStream *stream, CamelMimePart *part,
 	CamelCipherContext *cipher;
 	CamelException ex;
 	const char *protocol;
+	int len;
 
 	/* Currently we only handle RFC2015-style PGP encryption. */
 	protocol = camel_content_type_param (((CamelDataWrapper *) part)->mime_type, "protocol");
@@ -1057,7 +1107,10 @@ emf_multipart_encrypted(EMFormat *emf, CamelStream *stream, CamelMimePart *part,
 		return;
 	}
 
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".encrypted");
 	em_format_part(emf, stream, mime_part);
+	g_string_truncate(emf->part_id, len);
 	camel_object_unref(mime_part);
 }
 
@@ -1076,7 +1129,8 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	CamelMimePart *body_part, *display_part = NULL;
 	CamelContentType *content_type;
 	const char *location, *start;
-	int i, nparts;
+	int i, nparts, partidlen, displayid = 0;
+	char *oldpartid;
 	CamelURL *base_save = NULL;
 	struct _EMFormatPURITree *ptree;
 	EMFormatPURI *puri, *purin;
@@ -1104,6 +1158,7 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 			
 			if (cid && !strncmp(cid, start, len) && strlen(cid) == len) {
 				display_part = body_part;
+				displayid = i;
 				break;
 			}
 		}
@@ -1125,16 +1180,24 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	}
 	em_format_push_level(emf);
 
+	oldpartid = g_strdup(emf->part_id->str);
+	partidlen = emf->part_id->len;
+
 	/* queue up the parts for possible inclusion */
 	for (i = 0; i < nparts; i++) {
 		body_part = camel_multipart_get_part(mp, i);
 		if (body_part != display_part) {
+			/* set the partid since add_puri uses it */
+			g_string_append_printf(emf->part_id, ".related.%d", i);
 			puri = em_format_add_puri(emf, sizeof(EMFormatPURI), NULL, body_part, emf_write_related);
+			g_string_truncate(emf->part_id, partidlen);
 			d(printf(" part '%s' '%s' added\n", puri->uri?puri->uri:"", puri->cid));
 		}
 	}
 	
+	g_string_append_printf(emf->part_id, ".related.%d", displayid);
 	em_format_part(emf, stream, display_part);
+	g_string_truncate(emf->part_id, partidlen);
 	camel_stream_flush(stream);
 
 	ptree = emf->pending_uri_level;
@@ -1143,14 +1206,19 @@ emf_multipart_related(EMFormat *emf, CamelStream *stream, CamelMimePart *part, c
 	while (purin) {
 		if (purin->use_count == 0) {
 			d(printf("part '%s' '%s' used '%d'\n", purin->uri?purin->uri:"", purin->cid, purin->use_count));
-			if (purin->func == emf_write_related)
+			if (purin->func == emf_write_related) {
+				g_string_printf(emf->part_id, "%s", puri->part_id);
 				em_format_part(emf, stream, puri->part);
-			else
+			} else
 				printf("unreferenced uri generated by format code: %s\n", purin->uri?purin->uri:purin->cid);
 		}
 		puri = purin;
 		purin = purin->next;
 	}
+
+	g_string_printf(emf->part_id, "%s", oldpartid);
+	g_free(oldpartid);
+
 	em_format_pull_level(emf);
 	
 	if (location) {
@@ -1169,6 +1237,7 @@ emf_multipart_signed(EMFormat *emf, CamelStream *stream, CamelMimePart *part, co
 	CamelException ex;
 	const char *message = NULL;
 	gboolean good = FALSE;
+	int len;
 
 	mps = (CamelMultipartSigned *)camel_medium_get_content_object((CamelMedium *)part);
 	if (!CAMEL_IS_MULTIPART_SIGNED(mps)
@@ -1177,7 +1246,10 @@ emf_multipart_signed(EMFormat *emf, CamelStream *stream, CamelMimePart *part, co
 		return;
 	}
 
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".signed");
 	em_format_part(emf, stream, cpart);
+	g_string_truncate(emf->part_id, len);
 
 	/* FIXME: This sequence is also copied in em-format-html.c */
 
@@ -1220,18 +1292,21 @@ emf_multipart_signed(EMFormat *emf, CamelStream *stream, CamelMimePart *part, co
 	camel_cipher_validity_free(valid);
 }
 
-/* this is only a fallback, any implementer should implement */
 static void
 emf_message_rfc822(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
 	CamelDataWrapper *dw = camel_medium_get_content_object((CamelMedium *)part);
+	int len;
 
 	if (!CAMEL_IS_MIME_MESSAGE(dw)) {
 		em_format_format_source(emf, stream, part);
 		return;
 	}
 
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".rfc822");
 	em_format_format_message(emf, stream, (CamelMedium *)dw);
+	g_string_truncate(emf->part_id, len);
 }
 
 static EMFormatHandler type_builtin_table[] = {
