@@ -211,7 +211,7 @@ mail_filter_folder (CamelFolder *source_folder, GPtrArray *uids,
 void
 mail_filter_on_demand (CamelFolder *folder, GPtrArray *uids)
 {
-	mail_filter_folder (folder, uids, FILTER_SOURCE_INCOMING, FALSE, NULL);
+	mail_filter_folder (folder, uids, FILTER_SOURCE_DEMAND, FALSE, NULL);
 }
 
 /* ********************************************************************** */
@@ -221,38 +221,17 @@ static char *
 uid_cachename_hack (CamelStore *store)
 {
 	CamelURL *url = CAMEL_SERVICE (store)->url;
-	char *encoded_url, *filename, *old_location;
-	struct stat st;
+	char *encoded_url, *filename;
+	const char *evolution_dir;
 	
-	encoded_url = g_strdup_printf ("pop://%s%s%s@%s/", url->user,
+	encoded_url = g_strdup_printf ("%s%s%s@%s", url->user,
 				       url->authmech ? ";auth=" : "",
 				       url->authmech ? url->authmech : "",
 				       url->host);
 	e_filename_make_safe (encoded_url);
 	
-	filename = g_strdup_printf ("%s/mail/pop3/cache-%s",
-				    mail_component_peek_base_directory (mail_component_peek ()), encoded_url);
-	
-	/* lame hack, but we can't expect user's to actually migrate
-           their cache files - brain power requirements are too
-           high. */
-	if (stat (filename, &st) == -1) {
-		/* This is either the first time the user has checked
-                   mail with this POP provider or else their cache
-                   file is in the old location... */
-		old_location = g_strdup_printf ("%s/config/cache-%s",
-						mail_component_peek_base_directory (mail_component_peek ()),
-						encoded_url);
-		if (stat (old_location, &st) == -1) {
-			/* old location doesn't exist either so use the new location */
-			g_free (old_location);
-		} else {
-			/* old location exists, so I guess we use the old cache file location */
-			g_free (filename);
-			filename = old_location;
-		}
-	}
-	
+	evolution_dir = mail_component_peek_base_directory (mail_component_peek ());
+	filename = g_build_filename (evolution_dir, "mail", "pop", encoded_url, "uid-cache", NULL);
 	g_free (encoded_url);
 	
 	return filename;
@@ -908,6 +887,10 @@ mail_append_mail (CamelFolder *folder, CamelMimeMessage *message, CamelMessageIn
 	g_assert(CAMEL_IS_FOLDER (folder));
 	g_assert(CAMEL_IS_MIME_MESSAGE (message));
 	
+	if (!camel_medium_get_header (CAMEL_MEDIUM (message), "X-Mailer"))
+		camel_medium_set_header (CAMEL_MEDIUM (message), "X-Mailer",
+					 "Ximian Evolution " VERSION SUB_VERSION " " VERSION_COMMENT);
+
 	m = mail_msg_new (&append_mail_op, NULL, sizeof (*m));
 	m->folder = folder;
 	camel_object_ref(folder);
@@ -1059,14 +1042,14 @@ get_folderinfo_desc (struct _mail_msg *mm, int done)
 	return ret;
 }
 
-static void
-add_vtrash_or_vjunk_info (CamelStore *store, CamelFolderInfo *info, gchar *name, gchar *full_name, gchar *url_base, gboolean unread_count)
+static CamelFolderInfo *
+add_special_info(CamelStore *store, CamelFolderInfo *info, char *name, char *full_name, char *url_base)
 {
 	CamelFolderInfo *fi, *vinfo, *parent;
 	char *uri, *path;
 	CamelURL *url;
 	
-	g_return_if_fail (info != NULL);
+	g_return_val_if_fail (info != NULL, NULL);
 
 	parent = NULL;
 	for (fi = info; fi; fi = fi->sibling) {
@@ -1107,22 +1090,10 @@ add_vtrash_or_vjunk_info (CamelStore *store, CamelFolderInfo *info, gchar *name,
 	vinfo->full_name = g_strdup (full_name);
 	vinfo->name = g_strdup(vinfo->full_name);
 	vinfo->url = g_strdup_printf ("%s:%s", url_base, uri);
-	if (!unread_count)
-		vinfo->unread_message_count = -1;
 	vinfo->path = g_strdup_printf("/%s", vinfo->name);
 	g_free (uri);
-}
 
-static void
-add_vtrash_info (CamelStore *store, CamelFolderInfo *info)
-{
-	add_vtrash_or_vjunk_info (store, info, CAMEL_VTRASH_NAME, _("Trash"), "vtrash", FALSE);
-}
-
-static void
-add_vjunk_info (CamelStore *store, CamelFolderInfo *info)
-{
-	add_vtrash_or_vjunk_info (store, info, CAMEL_VJUNK_NAME, _("Junk"), "vjunk", TRUE);
+	return vinfo;
 }
 
 static void
@@ -1151,9 +1122,14 @@ get_folderinfo_get (struct _mail_msg *mm)
 	m->info = camel_store_get_folder_info (m->store, NULL, flags, &mm->ex);
 	if (m->info) {
 		if (m->info->url && (m->store->flags & CAMEL_STORE_VTRASH))
-			add_vtrash_info(m->store, m->info);
-		if (m->info->url && (m->store->flags & CAMEL_STORE_VJUNK))
-			add_vjunk_info(m->store, m->info);
+			add_special_info(m->store, m->info, CAMEL_VTRASH_NAME, _("Trash"), "vtrash");
+		if (m->info->url && (m->store->flags & CAMEL_STORE_VJUNK)) {
+			CamelFolderInfo *info;
+
+			info = add_special_info(m->store, m->info, CAMEL_VJUNK_NAME, _("Junk"), "vjunk");
+			info->unread_message_count = -1;
+		}
+
 		if (CAMEL_IS_VEE_STORE(m->store))
 			add_unmatched_info(m->info);
 	}
