@@ -28,7 +28,6 @@
 
 #include "e-util/e-dialog-utils.h"
 
-#include "e-setup.h"
 #include "e-shell-constants.h"
 #include "e-shell-offline-handler.h"
 #include "e-shell-settings-dialog.h"
@@ -478,7 +477,6 @@ e_shell_construct (EShell *shell,
 		   const char *iid,
 		   EShellStartupLineMode startup_line_mode)
 {
-	GtkWidget *splash = NULL;
 	EShellPrivate *priv;
 	CORBA_Object corba_object;
 	gboolean start_online;
@@ -502,10 +500,7 @@ e_shell_construct (EShell *shell,
 		return E_SHELL_CONSTRUCT_RESULT_CANNOTREGISTER;
 
 	while (gtk_events_pending ())
-		gtk_main_iteration ();
-	
-	if (splash)
-		gtk_widget_destroy (splash);
+		gtk_main_iteration ();	
 	
 	if (e_shell_startup_wizard_create () == FALSE) {
 		bonobo_object_unref (BONOBO_OBJECT (shell));
@@ -581,23 +576,19 @@ e_shell_new (EShellStartupLineMode startup_line_mode,
  * upgrade from @from_version is unsupported).
  **/
 gboolean
-e_shell_attempt_upgrade (EShell *shell,
-			 const char *from_version)
+e_shell_attempt_upgrade (EShell *shell, int major, int minor, int revision)
 {
 	GSList *component_infos, *p;
-	int major, minor, revision;
 	int current_major, current_minor, current_revision;
 	gboolean success;
 
 	g_return_val_if_fail (E_IS_SHELL (shell), FALSE);
-	g_return_val_if_fail (from_version != NULL, FALSE);
 
-	sscanf (from_version, "%u.%u.%u", &major, &minor, &revision);
 	sscanf (VERSION, "%u.%u.%u", &current_major, &current_minor, &current_revision);
 
 	if (! (current_major > major
 	       || (current_major == major && current_minor > minor)
-	       || (current_minor == current_minor && current_revision > current_revision))) {
+	       || (current_minor == current_minor && current_revision > revision))) {
 		return TRUE;
 	}
 
@@ -624,16 +615,16 @@ e_shell_attempt_upgrade (EShell *shell,
 			}
 
 			exception_text = bonobo_exception_get_text (&ev);
-			g_warning ("Upgrade of component \"%s\" from version %s failed with exception %s",
-				   info->alias, from_version, exception_text);
+			g_warning ("Upgrade of component \"%s\" from version %d.%d.%d failed with exception %s",
+				   info->alias, major, minor, revision, exception_text);
 			g_free (exception_text);
 			CORBA_exception_free (&ev);
 			success = FALSE;
 		} else {
 			CORBA_exception_free (&ev);
 			if (! component_upgraded) {
-				g_warning ("Component \"%s\" could not upgrade configuration from version \"%s\"",
-					   info->alias, from_version);
+				g_warning ("Component \"%s\" could not upgrade configuration from version %d.%d.%d",
+					   info->alias, major, minor, revision);
 				success = FALSE;
 			}
 		}
@@ -914,7 +905,6 @@ e_shell_go_online (EShell *shell,
 		EComponentInfo *info = p->data;
 		CORBA_Environment ev;
 		GNOME_Evolution_Offline offline_interface;
-		const char *id;
 
 		CORBA_exception_init (&ev);
 
@@ -928,7 +918,7 @@ e_shell_go_online (EShell *shell,
 
 		GNOME_Evolution_Offline_goOnline (offline_interface, &ev);
 		if (ev._major != CORBA_NO_EXCEPTION)
-			g_warning ("Error putting component `%s' online.", id);
+			g_warning ("Error putting component `%s' online.", info->id);
 
 		CORBA_exception_free (&ev);
 	}
