@@ -20,21 +20,27 @@
  * Author: Ettore Perazzoli
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include "e-util/e-dialog-utils.h"
 #include "e-util/e-gtk-utils.h"
+#include "e-util/e-bconf-map.h"
 
 #include "e-icon-factory.h"
 #include "e-shell-constants.h"
-#include "e-shell-config.h"
-#include "e-setup.h"
 
 #include "e-shell.h"
 
+#include <libxml/xmlmemory.h>
+#include <libxml/parser.h>
+#include <libxml/tree.h>
+
 #include <gconf/gconf-client.h>
 
 #include <gtk/gtkalignment.h>
+#include <gtk/gtkbox.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
@@ -60,11 +66,7 @@
 #include <glade/glade.h>
 
 #include "e-config-upgrade.h"
-#include "Evolution-Wombat.h"
-
-#ifdef GTKHTML_HAVE_GCONF
-#include <gconf/gconf.h>
-#endif
+#include "Evolution-DataServer.h"
 
 #include <gal/widgets/e-cursors.h>
 
@@ -77,21 +79,22 @@
 
 #include <pthread.h>
 
-
+
+#define DEVELOPMENT_WARNING
+
+
 static EShell *shell = NULL;
-static char *evolution_directory = NULL;
 
 /* Command-line options.  */
-static gboolean no_splash = FALSE;
 static gboolean start_online = FALSE;
 static gboolean start_offline = FALSE;
-static gboolean force_upgrade = FALSE;
 static gboolean setup_only = FALSE;
 static gboolean killev = FALSE;
+static char *default_component_id = NULL;
+
+static char *evolution_debug_log = NULL;
 
-extern char *evolution_debug_log;
 
-
 static GtkWidget *
 quit_box_new (void)
 {
@@ -141,19 +144,13 @@ quit_box_new (void)
 }
 
 static void
-no_views_left_cb (EShell *shell, gpointer data)
+no_windows_left_cb (EShell *shell, gpointer data)
 {
 	GtkWidget *quit_box;
 
 	quit_box = quit_box_new ();
 	g_object_add_weak_pointer (G_OBJECT (quit_box), (void **) &quit_box);
 
-	/* FIXME: This is wrong.  We should exit only when the shell is
-	   destroyed.  But refcounting is broken at present, so this is a
-	   reasonable workaround for now.  */
-
-	e_shell_unregister_all (shell);
-
 	bonobo_object_unref (BONOBO_OBJECT (shell));
 
 	if (quit_box != NULL)
@@ -169,43 +166,48 @@ shell_weak_notify (void *data,
 	bonobo_main_quit ();
 }
 
-
-static void
-kill_wombat (void)
-{
-	g_print ("(Killing old version of Wombat...)\n");
 
-	system (KILL_PROCESS_CMD " -9 lt-evolution-wombat 2> /dev/null");
-	system (KILL_PROCESS_CMD " -9 evolution-wombat 2> /dev/null");
+#ifdef KILL_PROCESS_CMD
+
+static void
+kill_dataserver (void)
+{
+	g_print ("(Killing old version of evolution-data-server...)\n");
+
+	system (KILL_PROCESS_CMD " -9 lt-evolution-data-server 2> /dev/null");
+	system (KILL_PROCESS_CMD " -9 evolution-data-server-1.0 2> /dev/null");
+
+	system (KILL_PROCESS_CMD " -9 lt-evolution-alarm-notify 2> /dev/null");
+	system (KILL_PROCESS_CMD " -9 evolution-alarm-notify 2> /dev/null");
 }
 
 static void
-kill_old_wombat (void)
+kill_old_dataserver (void)
 {
-	GNOME_Evolution_WombatInterfaceCheck iface;
+	GNOME_Evolution_DataServer_InterfaceCheck iface;
 	CORBA_Environment ev;
 	CORBA_char *version;
 
 	CORBA_exception_init (&ev);
 
-	iface = bonobo_activation_activate_from_id ("OAFIID:GNOME_Evolution_Wombat_InterfaceCheck", 0, NULL, &ev);
+	iface = bonobo_activation_activate_from_id ("OAFIID:GNOME_Evolution_DataServer_InterfaceCheck", 0, NULL, &ev);
 	if (BONOBO_EX (&ev) || iface == CORBA_OBJECT_NIL) {
-		kill_wombat ();
+		kill_dataserver ();
 		CORBA_exception_free (&ev);
 		return;
 	}
 
-	version = GNOME_Evolution_WombatInterfaceCheck__get_interfaceVersion (iface, &ev);
+	version = GNOME_Evolution_DataServer_InterfaceCheck__get_interfaceVersion (iface, &ev);
 	if (BONOBO_EX (&ev)) {
-		kill_wombat ();
+		kill_dataserver ();
 		CORBA_Object_release (iface, &ev);
 		CORBA_exception_free (&ev);
 		return;
 	}
 
-	if (strcmp (version, VERSION) != 0) {
+	if (strcmp (version, DATASERVER_VERSION) != 0) {
 		CORBA_free (version);
-		kill_wombat ();
+		kill_dataserver ();
 		CORBA_Object_release (iface, &ev);
 		CORBA_exception_free (&ev);
 		return;
@@ -215,8 +217,11 @@ kill_old_wombat (void)
 	CORBA_Object_release (iface, &ev);
 	CORBA_exception_free (&ev);
 }
+#endif
+
+
+#ifdef DEVELOPMENT_WARNING
 
-
 /* Warning dialog to scare people off a little bit.  */
 
 static void
@@ -278,7 +283,7 @@ show_development_warning (GtkWindow *parent)
                   "\n"
 		  "We hope that you enjoy the results of our hard work, and we\n"
 		  "eagerly await your contributions!\n"),
-		"1.2.x (1.2.2)");
+		"1.4");
 	label = gtk_label_new (text);
 	g_free(text);
 
@@ -315,25 +320,121 @@ show_development_warning (GtkWindow *parent)
    soon as the first view is created.  */
 
 static void
-view_map_callback (GtkWidget *widget,
-		   void *data)
+window_map_callback (GtkWidget *widget,
+		     void *data)
 {
-	g_signal_handlers_disconnect_by_func (widget, G_CALLBACK (view_map_callback), data);
+	g_signal_handlers_disconnect_by_func (widget, G_CALLBACK (window_map_callback), data);
 
 	show_development_warning (GTK_WINDOW (widget));
 }
 
 static void
-new_view_created_callback (EShell *shell,
-			   EShellView *view,
-			   void *data)
+new_window_created_callback (EShell *shell,
+			     EShellWindow *window,
+			     void *data)
 {
-	g_signal_handlers_disconnect_by_func (shell, G_CALLBACK (new_view_created_callback), data);
+	g_signal_handlers_disconnect_by_func (shell, G_CALLBACK (new_window_created_callback), data);
 
-	g_signal_connect (view, "map", G_CALLBACK (view_map_callback), NULL);
+	g_signal_connect (window, "map", G_CALLBACK (window_map_callback), NULL);
+}
+
+#endif /* DEVELOPMENT_WARNING */
+
+static gboolean
+detect_version (GConfClient *gconf, int *major, int *minor, int *revision)
+{
+	char *val, *evolution_dir, *filename;
+	struct stat st;
+	
+	evolution_dir = g_build_filename (g_get_home_dir (), "evolution", NULL);
+	filename = g_build_filename (evolution_dir, "config.xmldb", NULL);
+	
+	val = gconf_client_get_string(gconf, "/apps/evolution/version", NULL);	
+	if (val) {
+		/* Since 1.4.0 We've been keeping the version key in gconf */
+		sscanf(val, "%u.%u.%u", major, minor, revision);
+		g_free(val);
+	} else if (lstat (filename, &st) != 0 || !S_ISDIR (st.st_mode)) {
+		/* If ~/evolution does not exit or is not a directory it must be a new installation */
+		*major = 0;
+		*minor = 0;
+		*revision = 0;
+	} else {
+		xmlDocPtr config_doc = NULL;
+		xmlNodePtr source;
+		char *tmp;
+		
+		if (lstat(filename, &st) == 0
+		    && S_ISREG(st.st_mode))
+			config_doc = xmlParseFile (filename);
+
+		tmp = NULL;
+		if (config_doc
+		    && (source = e_bconf_get_path (config_doc, "/Shell"))
+		    && (tmp = e_bconf_get_value (source, "upgrade_from_1_0_to_1_2_performed"))
+		    && tmp[0] == '1' ) {
+			*major = 1;
+			*minor = 2;
+			*revision = 0;
+		} else {
+			*major = 1;
+			*minor = 0;
+			*revision = 0;
+		}
+		if (tmp)
+			xmlFree(tmp);
+		if (config_doc)
+			xmlFreeDoc (config_doc);		
+	}
+
+	g_free (evolution_dir);
+	g_free (filename);
+
+	return TRUE;
+}
+
+static void
+attempt_upgrade (EShell *shell)
+{
+	GConfClient *gconf_client;
+	int major = 0, minor = 0, revision = 0;
+
+	gconf_client = gconf_client_get_default ();
+
+	if (!detect_version (gconf_client, &major, &minor, &revision)
+	    || !e_shell_attempt_upgrade (shell, major, minor, revision)) 
+		e_notice (NULL, GTK_MESSAGE_ERROR,
+			  _("Warning: Evolution could not upgrade all your data from version %d.%d.%d.\n"
+			    "The data hasn't been deleted, but it will not be seen by this version of Evolution.\n"),
+			  major, minor, revision);
+
+	gconf_client_set_string (gconf_client, "/apps/evolution/version", VERSION, NULL);
+	g_object_unref (gconf_client);
+}
+
+
+static void
+open_uris (GNOME_Evolution_Shell corba_shell, GSList *uri_list)
+{
+	GSList *p;
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
+
+	for (p = uri_list; p != NULL; p = p->next) {
+		const char *uri;
+
+		uri = (const char *) p->data;
+		GNOME_Evolution_Shell_handleURI (corba_shell, uri, &ev);
+		if (ev._major != CORBA_NO_EXCEPTION) {
+			g_warning ("Invalid URI: %s", uri);
+			CORBA_exception_free (&ev);
+		}
+	}
+
+	CORBA_exception_free (&ev);
 }
 
-
 /* This is for doing stuff that requires the GTK+ loop to be running already.  */
 
 static gint
@@ -344,12 +445,11 @@ idle_cb (void *data)
 	CORBA_Environment ev;
 	EShellConstructResult result;
 	EShellStartupLineMode startup_line_mode;
-	GSList *p;
 	gboolean have_evolution_uri;
-	gboolean display_default;
-	gboolean displayed_any;
 
-	kill_old_wombat ();
+#ifdef KILL_PROCESS_CMD
+	kill_old_dataserver ();
+#endif
 
 	CORBA_exception_init (&ev);
 
@@ -362,19 +462,18 @@ idle_cb (void *data)
 	else
 		startup_line_mode = E_SHELL_STARTUP_LINE_MODE_OFFLINE;
 
-	shell = e_shell_new (evolution_directory, ! no_splash, startup_line_mode, &result);
-	g_free (evolution_directory);
+	shell = e_shell_new (startup_line_mode, &result);
 
 	switch (result) {
 	case E_SHELL_CONSTRUCT_RESULT_OK:
-		e_shell_config_factory_register (shell);
-
-		g_signal_connect (shell, "no_views_left", G_CALLBACK (no_views_left_cb), NULL);
+		g_signal_connect (shell, "no_windows_left", G_CALLBACK (no_windows_left_cb), NULL);
 		g_object_weak_ref (G_OBJECT (shell), shell_weak_notify, NULL);
 
+#ifdef DEVELOPMENT_WARNING
 		if (!getenv ("EVOLVE_ME_HARDER"))
-			g_signal_connect (shell, "new_view_created",
-					  G_CALLBACK (new_view_created_callback), NULL);
+			g_signal_connect (shell, "new_window_created",
+					  G_CALLBACK (new_window_created_callback), NULL);
+#endif
 
 		corba_shell = bonobo_object_corba_objref (BONOBO_OBJECT (shell));
 		corba_shell = CORBA_Object_duplicate (corba_shell, &ev);
@@ -401,63 +500,31 @@ idle_cb (void *data)
 
 	}
 
+	if (shell != NULL)
+		attempt_upgrade (shell);
+
 	have_evolution_uri = FALSE;
-	displayed_any = FALSE;
 
-	for (p = uri_list; p != NULL; p = p->next) {
-		const char *uri;
-
-		uri = (const char *) p->data;
-		if (strncmp (uri, E_SHELL_URI_PREFIX, E_SHELL_URI_PREFIX_LEN) == 0 ||
-		    strncmp (uri, E_SHELL_DEFAULTURI_PREFIX, E_SHELL_DEFAULTURI_PREFIX_LEN) == 0)
-			have_evolution_uri = TRUE;
-	}
-
-	if (shell == NULL) {
-		/* We're talking to a remote shell. If the user didn't ask us to open any particular
- 		   URI, then open another view of the default URI.  */
-		if (uri_list == NULL)
-			display_default = TRUE;
-		else
-			display_default = FALSE;
+	if (shell != NULL) {
+		e_shell_create_window (shell, default_component_id, NULL);
+		open_uris (corba_shell, uri_list);
 	} else {
-		/* We're starting a new shell. If the user didn't specify any evolution: URIs to
-		   view, AND we can't load the user's previous settings, then show the default
-		   URI.  */
-		if (! have_evolution_uri) {
-			e_shell_create_view (shell, NULL, NULL);
-			display_default = TRUE;
-			displayed_any = TRUE;
-		} else {
-			display_default = FALSE;
-		}
-	}
+		CORBA_Environment ev;
 
-	for (p = uri_list; p != NULL; p = p->next) {
-		const char *uri;
+		CORBA_exception_init (&ev);
+		if (uri_list != NULL)
+			open_uris (corba_shell, uri_list);
+		else
+			if (default_component_id == NULL)
+				GNOME_Evolution_Shell_createNewWindow (corba_shell, "", &ev);
+			else
+				GNOME_Evolution_Shell_createNewWindow (corba_shell, default_component_id, &ev);
 
-		uri = (const char *) p->data;
-		GNOME_Evolution_Shell_handleURI (corba_shell, uri, &ev);
-		if (ev._major == CORBA_NO_EXCEPTION)
-			displayed_any = TRUE;
-		else {
-			g_warning ("CORBA exception %s when requesting URI -- %s",
-				   BONOBO_EX_REPOID (&ev), uri);
-			CORBA_exception_free (&ev);
-		}
+		CORBA_exception_free (&ev);
 	}
 
 	g_slist_free (uri_list);
 
-	if (display_default && ! displayed_any) {
-		const char *uri;
-
-		uri = E_SHELL_VIEW_DEFAULT_URI;
-		GNOME_Evolution_Shell_handleURI (corba_shell, uri, &ev);
-		if (ev._major != CORBA_NO_EXCEPTION)
-			g_warning ("CORBA exception %s when requesting URI -- %s", BONOBO_EX_REPOID (&ev), uri);
-	}
-
 	CORBA_Object_release (corba_shell, &ev);
 
 	CORBA_exception_free (&ev);
@@ -524,8 +591,8 @@ int
 main (int argc, char **argv)
 {
 	struct poptOption options[] = {
-		{ "no-splash", '\0', POPT_ARG_NONE, &no_splash, 0,
-		  N_("Disable splash screen"), NULL },
+		{ "component", 'c', POPT_ARG_STRING, &default_component_id, 0,
+		  N_("Start Evolution activating the specified component"), NULL },
 		{ "offline", '\0', POPT_ARG_NONE, &start_offline, 0, 
 		  N_("Start in offline mode"), NULL },
 		{ "online", '\0', POPT_ARG_NONE, &start_online, 0, 
@@ -536,10 +603,6 @@ main (int argc, char **argv)
 #endif
 		{ "debug", '\0', POPT_ARG_STRING, &evolution_debug_log, 0, 
 		  N_("Send the debugging output of all components to a file."), NULL },
-#if 0
-		{ "force-upgrade", '\0', POPT_ARG_NONE, &force_upgrade, 0, 
-		  N_("Force upgrading of configuration files from Evolution 1.0.x"), NULL },
-#endif
 		{ "setup-only", '\0', POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN,
 		  &setup_only, 0, NULL, NULL },
 		POPT_AUTOHELP
@@ -550,6 +613,7 @@ main (int argc, char **argv)
 	GnomeProgram *program;
 	poptContext popt_context;
 	const char **args;
+	char *evolution_directory;
 
 	/* Make ElectricFence work.  */
 	free (malloc (10));
@@ -596,14 +660,13 @@ main (int argc, char **argv)
 
 	gnome_window_icon_set_default_from_file (EVOLUTION_IMAGES "/evolution-inbox.png");
 
-	/* FIXME */
+	/* FIXME We shouldn't be using the old directory at all I think */
 	evolution_directory = g_build_filename (g_get_home_dir (), "evolution", NULL);
-
-	if (! e_setup (evolution_directory))
-		exit (1);
 	if (setup_only)
 		exit (0);
 
+	g_free (evolution_directory);
+
 	uri_list = NULL;
 
 	g_value_init (&popt_context_value, G_TYPE_POINTER);
@@ -619,9 +682,7 @@ main (int argc, char **argv)
 	uri_list = g_slist_reverse (uri_list);
 	g_value_unset (&popt_context_value);
 
-	e_config_upgrade(evolution_directory);
-
-	gtk_idle_add (idle_cb, uri_list);
+	g_idle_add (idle_cb, uri_list);
 
 	bonobo_main ();
 
